/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */

#include "mainwindow.h"
#include "combinechannelsdialog.h"

static GtkObjectClass *parent_class;
static sample_t *combination_matrix;
static sample_t *samples = NULL;

static gboolean combine_channels_dialog_apply_proc(void *in, guint sample_size,
						   chunk_writeout_func out_func, WriteoutID id,
						   Chunk *chunk)
{
     sample_t *s = (sample_t *)in, d;
     guint i,j,chn=chunk->format.channels;
     if (!samples)
	  samples = g_malloc(sample_size);
     for (i=0; i<chn; i++) {
	  d = 0.0;
	  for (j=0; j<chn; j++)
	       d += s[j] * combination_matrix[i*chn+j];
	  samples[i] = d;
     }
     return out_func(id,samples,sample_size);
}

static void combine_channels_dialog_apply(EffectDialog *ed)
{
     guint i;
     CombineChannelsDialog *ccd = COMBINE_CHANNELS_DIALOG(ed);
     g_assert(ccd->channels == ed->mw->view->chunk->format.channels);
     for (i=0; i<ccd->channels*ccd->channels; i++)
	  if (floatbox_check(ccd->combination_matrix[i])) return;
     combination_matrix =g_malloc(ccd->channels*ccd->channels*sizeof(sample_t));
     for (i=0; i<ccd->channels*ccd->channels; i++)
	  combination_matrix[i] = ccd->combination_matrix[i]->val / 100.0;
     mainwindow_effect(ed->mw,combine_channels_dialog_apply_proc,NULL,TRUE,TRUE);
     g_free(samples);
     samples = NULL;
     g_free(combination_matrix);
}

static void combine_channels_dialog_chunk_changed(ChunkView *cv,
						  CombineChannelsDialog *ccd) 
{
    GtkWidget *w;
    if (ccd->channels != cv->chunk->format.channels) {
	w = combine_channels_dialog_new(EFFECT_DIALOG(ccd)->mw);
	gtk_widget_show(w);
	gtk_widget_destroy(GTK_WIDGET(ccd));
    }
}

static void combine_channels_mainwindow_set(EffectDialog *ed)
{
    int i,j;
    GtkWidget *a,*b,*c;
    gchar buf[20];
    CombineChannelsDialog *ccd = COMBINE_CHANNELS_DIALOG(ed);
    ccd->channels = ed->mw->view->chunk->format.channels;
    ccd->combination_matrix = g_malloc(ccd->channels*ccd->channels*sizeof(Floatbox *));
    for (i=0; i<ccd->channels*ccd->channels; i++)
        ccd->combination_matrix[i] = 
        FLOATBOX(floatbox_new( (i%(ccd->channels+1)==0) ? 100.0 : 0.0 ));
    for (i=0; i<ccd->channels; i++) {
        g_snprintf(buf,sizeof(buf),"Channel %d",i+1);
        a = gtk_frame_new(buf);
        gtk_box_pack_start(GTK_BOX(ed->input_area),a,FALSE,FALSE,0);
        gtk_widget_show(a);
        b = gtk_table_new(ccd->channels,3,FALSE);
        gtk_container_add(GTK_CONTAINER(a),b);
        gtk_widget_show(b);
        for (j=0; j<ccd->channels; j++) {
            g_snprintf(buf,sizeof(buf),"Channel %d",j+1);
            c = gtk_label_new(buf);
            gtk_table_attach(GTK_TABLE(b),c,0,1,j,j+1,GTK_FILL,0,0,0);
            gtk_widget_show(c);
            c = GTK_WIDGET(ccd->combination_matrix[i*ccd->channels+j]);
            gtk_table_attach(GTK_TABLE(b),c,1,2,j,j+1,GTK_FILL,0,0,0);
            gtk_widget_show(c);
            c = gtk_label_new("%");
            gtk_table_attach(GTK_TABLE(b),c,2,3,j,j+1,GTK_FILL,0,0,0);
            gtk_widget_show(c);
        }
    }
    gtk_signal_connect_while_alive(
	 GTK_OBJECT(ed->mw->view),"chunk-changed",
	 GTK_SIGNAL_FUNC(combine_channels_dialog_chunk_changed),
	 GTK_OBJECT(ed),GTK_OBJECT(ed));
}

static void combine_channels_destroy(GtkObject *obj)
{
    CombineChannelsDialog *ccd = COMBINE_CHANNELS_DIALOG(obj);
    g_free(ccd->combination_matrix);
    ccd->combination_matrix = NULL;
    parent_class->destroy(obj);
}

static void combine_channels_dialog_class_init(EffectDialogClass *klass)
{
    parent_class = gtk_type_class(effect_dialog_get_type());
    klass->apply = combine_channels_dialog_apply;
    klass->mainwindow_set = combine_channels_mainwindow_set;
    GTK_OBJECT_CLASS(klass)->destroy = combine_channels_destroy;
}

static void combine_channels_dialog_init(CombineChannelsDialog *obj)
{
    gtk_window_set_title(GTK_WINDOW(obj),"Combine channels");
}

GtkType combine_channels_dialog_get_type(void)
{
    static GtkType id = 0;
    if (!id) {
        GtkTypeInfo info = {
            "CombineChannelsDialog",
            sizeof(CombineChannelsDialog),
            sizeof(CombineChannelsDialogClass),
            (GtkClassInitFunc)combine_channels_dialog_class_init,
            (GtkObjectInitFunc)combine_channels_dialog_init
        };
        id = gtk_type_unique(effect_dialog_get_type(),&info);
    }
    return id;
}

GtkWidget *combine_channels_dialog_new(Mainwindow *mw)
{
    GtkWidget *w = gtk_type_new(combine_channels_dialog_get_type());
    effect_dialog_set_mainwindow(EFFECT_DIALOG(w),mw);
    return w;
}
