/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include "sound.h"
#include "inifile.h"
#include "main.h"
#include "configdialog.h"
#include "um.h"
#include "mainwindow.h"

static gchar *time_display_strings[] = { "MM:SS.t", "MM:SS.mmmm", "Samples" };

static void config_dialog_class_init(GtkObjectClass *klass)
{
}

static void config_dialog_ok(GtkButton *button, gpointer user_data)
{
    gchar *c;
    guint i;
    gboolean b = FALSE;
    ConfigDialog *cd = CONFIG_DIALOG(user_data);
    if (intbox_check(cd->sound_buffer_size) || 
        intbox_check(cd->disk_threshold) ||
	intbox_check(cd->view_quality) )
        return;
    c = (gchar *)gtk_entry_get_text(GTK_ENTRY(cd->sound_driver->entry));
    c = sound_driver_id_from_name(c);
    g_assert(c != NULL);

    b = b || inifile_set(INI_SETTING_SOUNDDRIVER,c);
    b = b || inifile_set_guint32(INI_SETTING_SOUNDBUFSIZE,
				 cd->sound_buffer_size->val);
    if (b) user_message("Some of the settings you have changed will not be "
			"activated until you restart the program", UM_OK);

    inifile_set_guint32(INI_SETTING_REALMAX,cd->disk_threshold->val);
    inifile_set_guint32(INI_SETTING_VIEW_QUALITY,
			cd->view_quality->val);
    inifile_set_gboolean(INI_SETTING_TIMESCALE,
			 gtk_toggle_button_get_active(cd->time_scale_default));
    inifile_set(INI_SETTING_MIXER,
		(char *)gtk_entry_get_text(cd->mixer_utility));

    for (i=0; strcmp(time_display_strings[i],
		     gtk_entry_get_text(GTK_ENTRY(cd->time_display->entry))); 
	 i++) {}
    get_time_mode = i;
    inifile_set_guint32(INI_SETTING_TIME_DISPLAY,i);

    sound_lock_driver = gtk_toggle_button_get_active(cd->sound_lock);
    inifile_set_gboolean("soundLock",sound_lock_driver);

    mainwindow_roll_cursor = gtk_toggle_button_get_active(cd->roll_cursor);
    inifile_set_gboolean("rollCursor",mainwindow_roll_cursor);

    gtk_widget_destroy(GTK_WIDGET(cd));

    mainwindow_update_texts();
}

static void sound_settings_click(GtkButton *button, gpointer user_data)
{
     ConfigDialog *cd = CONFIG_DIALOG(user_data);
     sound_driver_show_preferences(
	  sound_driver_id_from_name(
	       (char *)gtk_entry_get_text(GTK_ENTRY(cd->sound_driver->entry))));
}

static void sound_driver_changed(GtkEditable *editable, gpointer user_data)
{
     ConfigDialog *cd = CONFIG_DIALOG(user_data);
     gtk_widget_set_sensitive(GTK_WIDGET(cd->sound_driver_prefs),
			      sound_driver_has_preferences(
				   sound_driver_id_from_name(
					(char *)gtk_entry_get_text(
					     GTK_ENTRY(editable)))));
}

static void config_dialog_init(ConfigDialog *cd)
{
    GtkWidget *w,*a,*b,*c;
    GList *l;
    GtkAccelGroup* ag;
    guint key,i;

    ag = gtk_accel_group_new();

    gtk_window_set_title(GTK_WINDOW(cd),"Preferences");
    gtk_window_set_position(GTK_WINDOW (cd), GTK_WIN_POS_MOUSE);
    gtk_container_set_border_width(GTK_CONTAINER(cd),5);

    /* Create the main widgets */

    w = intbox_new( inifile_get_guint32( INI_SETTING_REALMAX, INI_SETTING_REALMAX_DEFAULT ));
    cd->disk_threshold = INTBOX(w);

    w = intbox_new(inifile_get_guint32(INI_SETTING_SOUNDBUFSIZE,
                                       INI_SETTING_SOUNDBUFSIZE_DEFAULT));
    cd->sound_buffer_size = INTBOX(w);

    w = gtk_combo_new();
    cd->sound_driver = GTK_COMBO(w);
    l = sound_driver_valid_names();
    gtk_combo_set_popdown_strings(cd->sound_driver,l);
    gtk_entry_set_text(GTK_ENTRY(cd->sound_driver->entry),sound_driver_name());
    gtk_editable_set_editable(GTK_EDITABLE(cd->sound_driver->entry),FALSE);
    gtk_signal_connect(GTK_OBJECT(cd->sound_driver->entry),"changed",
		       GTK_SIGNAL_FUNC(sound_driver_changed),cd);

    w = gtk_entry_new();
    cd->mixer_utility = GTK_ENTRY(w);
    gtk_entry_set_text(cd->mixer_utility,inifile_get(INI_SETTING_MIXER,
						     INI_SETTING_MIXER_DEFAULT));


    w = intbox_new(inifile_get_guint32(INI_SETTING_VIEW_QUALITY,
				       INI_SETTING_VIEW_QUALITY_DEFAULT));
    cd->view_quality = INTBOX(w);

    w = gtk_check_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL (GTK_BIN (w)->child), 
				"Show _time scale by default");
    gtk_widget_add_accelerator (w, "clicked", ag, key, GDK_MOD1_MASK, 
				(GtkAccelFlags) 0);
    cd->time_scale_default = GTK_TOGGLE_BUTTON(w);
    gtk_toggle_button_set_active(cd->time_scale_default,inifile_get_gboolean(
				      INI_SETTING_TIMESCALE, 
				      INI_SETTING_TIMESCALE_DEFAULT));

    w = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(w)->child), "_Settings");
    gtk_widget_add_accelerator(w, "clicked", ag, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
    gtk_signal_connect(GTK_OBJECT(w),"clicked",
		       GTK_SIGNAL_FUNC(sound_settings_click),cd);
    gtk_widget_set_sensitive(w,sound_driver_has_preferences(NULL));
    cd->sound_driver_prefs = GTK_BUTTON(w);

    w = gtk_check_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL (GTK_BIN (w)->child), 
				"_Keep sound driver opened (to avoid start/stop"
				" clicks)");
    gtk_widget_add_accelerator (w, "clicked", ag, key, GDK_MOD1_MASK, 
				(GtkAccelFlags) 0);
    cd->sound_lock = GTK_TOGGLE_BUTTON(w);
    gtk_toggle_button_set_active(cd->sound_lock,sound_lock_driver);

    w = gtk_check_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL (GTK_BIN (w)->child), 
				"_Update cursor information while playing");
    gtk_widget_add_accelerator (w, "clicked", ag, key, GDK_MOD1_MASK, 
				(GtkAccelFlags) 0);
    cd->roll_cursor = GTK_TOGGLE_BUTTON(w);
    gtk_toggle_button_set_active(cd->roll_cursor,mainwindow_roll_cursor);

    
    w = gtk_combo_new();
    cd->time_display = GTK_COMBO(w);
    l = NULL;
    for (i=0; i<ARRAY_LENGTH(time_display_strings); i++) 
	 l = g_list_append(l,time_display_strings[i]);
    gtk_combo_set_popdown_strings(cd->time_display,l);
    i = inifile_get_guint32(INI_SETTING_TIME_DISPLAY,0);
    if (i>=ARRAY_LENGTH(time_display_strings)) i=0;
    gtk_entry_set_text(GTK_ENTRY(cd->time_display->entry),
		       time_display_strings[i]);
    gtk_editable_set_editable(GTK_EDITABLE(cd->time_display->entry),FALSE);

    /* Layout the window */

    a = gtk_vbox_new(FALSE,5);
    gtk_container_add(GTK_CONTAINER(cd),a);
    gtk_widget_show(a);
    b = gtk_table_new(11,3,FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(b), 4);
    gtk_table_set_col_spacings(GTK_TABLE(b), 4);
    gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
    gtk_widget_show(b);

    c = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL (c), "_Memory editing limit:");
    gtk_misc_set_alignment(GTK_MISC(c),0.0,0.5);
    gtk_table_attach(GTK_TABLE(b),c,0,1,0,1,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->disk_threshold);
    gtk_widget_add_accelerator(c, "grab_focus", ag, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
    gtk_table_attach(GTK_TABLE(b),c,1,2,0,1,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = gtk_label_new("bytes");
    gtk_table_attach(GTK_TABLE(b),c,2,3,0,1,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    c = gtk_hseparator_new();
    gtk_table_attach(GTK_TABLE(b),c,0,3,1,2,GTK_FILL,0,0,3);
    gtk_widget_show(c);
    c = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL (c), "Sound _driver:");
    gtk_misc_set_alignment(GTK_MISC(c),0.0,0.5);
    gtk_table_attach(GTK_TABLE(b),c,0,1,2,3,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->sound_driver);
    gtk_widget_add_accelerator(c, "grab_focus", ag, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
    gtk_table_attach(GTK_TABLE(b),c,1,2,2,3,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->sound_driver_prefs);
    gtk_table_attach(GTK_TABLE(b),c,2,3,2,3,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    c = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL (c), "Sound _buffer size:");
    gtk_misc_set_alignment(GTK_MISC(c),0.0,0.5);
    gtk_table_attach(GTK_TABLE(b),c,0,1,3,4,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->sound_buffer_size);
    gtk_widget_add_accelerator(c, "grab_focus", ag, key, GDK_MOD1_MASK, 
			       (GtkAccelFlags) 0);
    gtk_table_attach(GTK_TABLE(b),c,1,2,3,4,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = gtk_label_new("bytes");
    gtk_table_attach(GTK_TABLE(b),c,2,3,3,4,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    c = GTK_WIDGET(cd->sound_lock);
    gtk_table_attach(GTK_TABLE(b),c,1,3,4,5,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    c = GTK_WIDGET(cd->roll_cursor);
    gtk_table_attach(GTK_TABLE(b),c,1,3,5,6,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    c = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL (c), "Mi_xer utility");
    gtk_misc_set_alignment(GTK_MISC(c),0.0,0.5);
    gtk_table_attach(GTK_TABLE(b),c,0,1,6,7,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->mixer_utility);
    gtk_widget_add_accelerator(c, "grab_focus", ag, key, GDK_MOD1_MASK, 
			       (GtkAccelFlags) 0);
    gtk_table_attach(GTK_TABLE(b),c,1,3,6,7,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    c = gtk_hseparator_new();
    gtk_table_attach(GTK_TABLE(b),c,0,3,7,8,GTK_FILL,0,0,3);
    gtk_widget_show(c);
    c = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL (c), "View _quality:");
    gtk_table_attach(GTK_TABLE(b),c,0,1,8,9,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->view_quality);
    gtk_widget_add_accelerator(c, "grab_focus", ag, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
    gtk_table_attach(GTK_TABLE(b),c,1,3,8,9,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->time_scale_default);
    gtk_table_attach(GTK_TABLE(b),c,1,3,9,10,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    
    c = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL(c), "_Time display:");
    gtk_table_attach(GTK_TABLE(b),c,0,1,10,11,GTK_FILL,0,0,0);
    gtk_widget_show(c);
    c = GTK_WIDGET(cd->time_display);
    gtk_table_attach(GTK_TABLE(b),c,1,3,10,11,GTK_FILL,0,0,0);
    gtk_widget_show(c);

    b = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
    gtk_widget_show(b);
    b = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(b), GTK_BUTTONBOX_END);
    gtk_box_pack_start(GTK_BOX(a),b,FALSE,TRUE,0);
    gtk_widget_show(b);

    c = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL (GTK_BIN (c)->child), "_OK");
    gtk_widget_add_accelerator (c, "clicked", ag, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
    gtk_signal_connect(GTK_OBJECT(c),"clicked",
    GTK_SIGNAL_FUNC(config_dialog_ok),cd);

    gtk_container_add (GTK_CONTAINER (b), c);
    GTK_WIDGET_SET_FLAGS (c, GTK_CAN_DEFAULT);
    gtk_widget_show(c);

    c = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL (GTK_BIN (c)->child), "_Close");
    gtk_widget_add_accelerator (c, "clicked", ag, key, GDK_MOD1_MASK, (GtkAccelFlags) 0);
    gtk_signal_connect_object(GTK_OBJECT(c),"clicked",
                              GTK_SIGNAL_FUNC(gtk_widget_destroy),
                              GTK_OBJECT(cd));
    gtk_container_add (GTK_CONTAINER (b), c);
    GTK_WIDGET_SET_FLAGS (c, GTK_CAN_DEFAULT);
    gtk_widget_show(c);

    gtk_window_add_accel_group(GTK_WINDOW (cd), ag);
}

GtkType config_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "ConfigDialog",
	       sizeof(ConfigDialog),
	       sizeof(ConfigDialogClass),
	       (GtkClassInitFunc) config_dialog_class_init,
	       (GtkObjectInitFunc) config_dialog_init
	  };
	  id = gtk_type_unique(gtk_window_get_type(),&info);
     }
     return id;
}

GtkWidget *config_dialog_new(void)
{
     return gtk_type_new(config_dialog_get_type());
}
