/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <math.h>
#include <gtk/gtk.h>
#include "mainwindow.h"
#include "effectdialog.h"
#include "main.h"

enum { APPLY_SIGNAL, MAINWINDOW_SET_SIGNAL, LAST_SIGNAL };
static guint effect_dialog_signals[LAST_SIGNAL] = { 0 };

static void effect_dialog_class_init(GtkObjectClass *klass)
{
     EFFECT_DIALOG_CLASS(klass)->apply = NULL;
     EFFECT_DIALOG_CLASS(klass)->mainwindow_set = NULL;

     effect_dialog_signals[APPLY_SIGNAL] =
	  gtk_signal_new("apply",GTK_RUN_FIRST,GTK_CLASS_TYPE(klass),
			 GTK_SIGNAL_OFFSET(EffectDialogClass,apply),
			 gtk_marshal_NONE__NONE,GTK_TYPE_NONE,0);
     effect_dialog_signals[MAINWINDOW_SET_SIGNAL] =
         gtk_signal_new("mainwindow_set",GTK_RUN_FIRST,GTK_CLASS_TYPE(klass),
                        GTK_SIGNAL_OFFSET(EffectDialogClass,mainwindow_set),
                        gtk_marshal_NONE__NONE,GTK_TYPE_NONE,0);

     gtk_object_class_add_signals(klass,effect_dialog_signals,LAST_SIGNAL);
}

static void apply_click(EffectDialog *ed)
{
     gtk_signal_emit(GTK_OBJECT(ed),effect_dialog_signals[APPLY_SIGNAL]);
}


static void effect_dialog_init(EffectDialog *v)
{
     GtkWidget *a,*b,*c;

     a = gtk_vbox_new ( FALSE, 5 );
     gtk_container_add(GTK_CONTAINER(v),a);
     gtk_widget_show(a);
     b = gtk_hbox_new ( FALSE, 3 );
     v->input_area = GTK_CONTAINER(b);
     gtk_box_pack_start ( GTK_BOX(a), b, FALSE, FALSE, 0 );
     gtk_widget_show ( b );
     b = gtk_hseparator_new ();
     gtk_box_pack_start ( GTK_BOX(a), b, FALSE, FALSE, 0 );
     gtk_widget_show ( b );
     b = gtk_hbox_new ( FALSE, 3 );
     v->button_area = GTK_CONTAINER(b);
     gtk_box_pack_start ( GTK_BOX(a), b, FALSE, FALSE, 0 );
     gtk_widget_show ( b );
     c = effect_dialog_add_button(v,"OK",GTK_SIGNAL_FUNC(apply_click));
     gtk_signal_connect_object(GTK_OBJECT(c),"clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(v));
     effect_dialog_add_button(v,"Apply",GTK_SIGNAL_FUNC(apply_click));
     effect_dialog_add_button(v,"Close",GTK_SIGNAL_FUNC(gtk_widget_destroy));

     gtk_container_set_border_width(GTK_CONTAINER(v),5);
}

GtkType effect_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "EffectDialog",
	       sizeof(EffectDialog),
	       sizeof(EffectDialogClass),
	       (GtkClassInitFunc) effect_dialog_class_init,
	       (GtkObjectInitFunc) effect_dialog_init
	  };
	  id = gtk_type_unique(gtk_window_get_type(),&info);
     }
     return id;
}

static void effect_dialog_chunk_change(ChunkView *view, gpointer user_data)
{
     if (view->chunk == NULL)
	  gtk_widget_destroy(GTK_WIDGET(user_data));
}

void effect_dialog_set_mainwindow(EffectDialog *ed, Mainwindow *mw)
{
     g_assert(ed->mw == NULL && mw != NULL);
     ed->mw = mw;
     gtk_signal_connect_object_while_alive(GTK_OBJECT(mw),"destroy",
					   GTK_SIGNAL_FUNC(gtk_widget_destroy),
					   GTK_OBJECT(ed));
     gtk_signal_connect_while_alive(GTK_OBJECT(mw->view),"chunk-changed",
				    GTK_SIGNAL_FUNC(effect_dialog_chunk_change),
				    ed,GTK_OBJECT(ed));
     gtk_signal_emit(GTK_OBJECT(ed),
                     effect_dialog_signals[MAINWINDOW_SET_SIGNAL]);
}

GtkWidget *effect_dialog_add_button(EffectDialog *ed, gchar *label, 
				    GtkSignalFunc callback)
{
     GtkWidget *a;
     a = gtk_button_new_with_label(label);
     gtk_signal_connect_object(GTK_OBJECT(a),"clicked",callback,GTK_OBJECT(ed));
     gtk_box_pack_start(GTK_BOX(ed->button_area),a,FALSE,FALSE,0);
     gtk_widget_show(a);
     return a;
}
