/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include "formatselector.h"


static void format_selector_class_init(GtkObjectClass *klass)
{     
}

static void format_selector_toggle(GtkToggleButton *button, gpointer user_data)
{
     FormatSelector *fs = FORMAT_SELECTOR(user_data);
     gint i;
     if (!gtk_toggle_button_get_active(button)) return;
     for (i=1; i<5; i++)
	  if (GTK_WIDGET(button) == fs->samplesize_widgets[i-1])
	       fs->samplesize = i;
     for (i=0; i<2; i++)
	  if (GTK_WIDGET(button) == fs->sign_widgets[i])
	       fs->sign = (gboolean)i;
}

static void format_selector_init(GtkWidget *widget)
{
     GtkTable *tbl = GTK_TABLE(widget);
     FormatSelector *fs = FORMAT_SELECTOR(widget);
     GtkWidget *w;
     GtkRadioButton *group=NULL;
     gchar c[8];
     gint i;

     gtk_table_resize(tbl,4,3);

     for (i=1; i<5; i++) {
	  g_snprintf(c,sizeof(c),"%d-bit",i*8);
	  w = gtk_radio_button_new_with_label_from_widget (group, c);
	  gtk_signal_connect(GTK_OBJECT(w),"toggled",
			     GTK_SIGNAL_FUNC(format_selector_toggle),fs);
	  if (!group) group = GTK_RADIO_BUTTON(w);
	  gtk_table_attach(tbl,w,0,1,i-1,i,GTK_EXPAND|GTK_FILL,0,0,0);
	  gtk_widget_show(w);
	  fs->samplesize_widgets[i-1] = w;
     }

     w = gtk_hseparator_new ();
     gtk_table_attach(tbl,w,1,2,0,4,0,0,0,0);
     gtk_widget_show(w);

     group = NULL;
     for (i=0;i<2;i++) {
	  w = gtk_radio_button_new_with_label_from_widget ( 
	       group, i?"Signed":"Unsigned" );
	  gtk_signal_connect(GTK_OBJECT(w),"toggled",
			     GTK_SIGNAL_FUNC(format_selector_toggle),fs);
	  if (!group) group = GTK_RADIO_BUTTON(w);
	  gtk_table_attach(tbl,w,2,3,i,i+1,GTK_EXPAND|GTK_FILL,0,0,0);
	  gtk_widget_show(w);
	  fs->sign_widgets[i] = w;
     }

     format_selector_set(fs,1,FALSE);
}

GtkType format_selector_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "FormatSelector",
	       sizeof(FormatSelector),
	       sizeof(FormatSelectorClass),
	       (GtkClassInitFunc) format_selector_class_init,
	       (GtkObjectInitFunc) format_selector_init
	  };
	  id = gtk_type_unique(gtk_table_get_type(),&info);
     }
     return id;
}

GtkWidget *format_selector_new(void)
{
     return gtk_type_new(format_selector_get_type());
}

void format_selector_set(FormatSelector *fs, guint samplesize, gboolean sign)
{
     if (samplesize <= 4 && samplesize >= 1) {
	  gtk_toggle_button_set_active(
	       GTK_TOGGLE_BUTTON(fs->samplesize_widgets[samplesize-1]),TRUE);
	  fs->samplesize = samplesize;
     }
     gtk_toggle_button_set_active(
	  GTK_TOGGLE_BUTTON(fs->sign_widgets[sign?1:0]),TRUE);
     fs->sign = sign;
}
