/*
 * Copyright (C) 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include "gotodialog.h"
#include "inifile.h"

static void goto_dialog_apply(EffectDialog *ed)
{
     guint32 p=0,q;
     gint32 o;
     ChunkView *v = ed->mw->view;
     int i;
     if (floatbox_check(GOTO_DIALOG(ed)->offset)) return;
     for (i=0; i<5; i++) {
	  if (gtk_toggle_button_get_active(GOTO_DIALOG(ed)->relbuttons[i]))
	       break;
     }
     switch (i) {
     case 0: p = 0; break;
     case 1: p = v->chunk->length-1; break;
     case 2: p = v->cursorpos; break;
     case 3: p = (v->selstart==v->selend)?0:v->selstart; break;
     case 4: p = (v->selstart==v->selend)?v->chunk->length-1:v->selend-1; break;
     default: g_assert_not_reached();
     }
     o = ((float)(v->chunk->format.samplerate))*GOTO_DIALOG(ed)->offset->val;
     q = p+o;
     if (q>=v->chunk->length) q=(o<0)?0:v->chunk->length-1;
     /* printf("i=%d p=%d o=%d q=%d\n",i,p,o,q); */
     chunk_view_set_cursor(v,q);

     inifile_set_gfloat("gotoOffset",GOTO_DIALOG(ed)->offset->val);
     inifile_set_guint32("gotoRel",i);
}

static void goto_dialog_init(GotoDialog *gd)
{
     GtkWidget *a,*b,*c;
     guint32 i;
     a = gtk_vbox_new(FALSE,0);
     gtk_box_pack_start(GTK_BOX(EFFECT_DIALOG(gd)->input_area),a,FALSE,FALSE,0);
     gtk_widget_show(a);
     b = gtk_hbox_new(FALSE,0);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     c = gtk_label_new("Place cursor ");
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = floatbox_new(inifile_get_gfloat("gotoOffset",0.0));
     gd->offset = FLOATBOX(c);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_label_new(" seconds");
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     b = gtk_radio_button_new_with_label(NULL,"after beginning of file");
     gd->relbuttons[0] = GTK_TOGGLE_BUTTON(b);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     b = gtk_radio_button_new_with_label_from_widget(
	  GTK_RADIO_BUTTON(b),"after end of file");
     gd->relbuttons[1] = GTK_TOGGLE_BUTTON(b);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     b = gtk_radio_button_new_with_label_from_widget(
	  GTK_RADIO_BUTTON(b),"after current cursor position");
     gd->relbuttons[2] = GTK_TOGGLE_BUTTON(b);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     b = gtk_radio_button_new_with_label_from_widget(
	  GTK_RADIO_BUTTON(b),"after selection start");
     gd->relbuttons[3] = GTK_TOGGLE_BUTTON(b);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     b = gtk_radio_button_new_with_label_from_widget(
	  GTK_RADIO_BUTTON(b),"after selection end");
     gd->relbuttons[4] = GTK_TOGGLE_BUTTON(b);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     b = gtk_label_new("(use a negative number to place the cursor before "
		       "instead of after the selected point)");
     gtk_label_set_justify(GTK_LABEL(b),GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap(GTK_LABEL(b),TRUE);
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);

     i = inifile_get_guint32("gotoRel",0);
     if (i>4) i=0;
     gtk_toggle_button_set_active(gd->relbuttons[i],TRUE);
     gtk_window_set_title(GTK_WINDOW(gd),"Position cursor");
}

static void goto_dialog_class_init(EffectDialogClass *klass)
{
     klass->apply = goto_dialog_apply;
}

GtkType goto_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "GotoDialog",
	       sizeof(GotoDialog),
	       sizeof(GotoDialogClass),
	       (GtkClassInitFunc) goto_dialog_class_init,
	       (GtkObjectInitFunc) goto_dialog_init
	  };
	  id = gtk_type_unique(effect_dialog_get_type(),&info);
     }
     return id;
}

GtkWidget *goto_dialog_new(Mainwindow *mw)
{
     GtkWidget *w;
     w = gtk_type_new(goto_dialog_get_type());
     effect_dialog_set_mainwindow(EFFECT_DIALOG(w),mw);
     return w;
}
