/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#ifndef GTKFILES_H_INCLUDED
#define GTKFILES_H_INCLUDED

#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>

typedef struct {
     FILE *handle;
     char *filename;
} EFILE;

#define EFILE_READ 0
#define EFILE_WRITE 1
#define EFILE_APPEND 2

gboolean xunlink(char *filename);
gboolean xlink(char *oldname, char *newname);

EFILE *e_fopen(char *filename, int mode);
gboolean e_fclose(EFILE *stream);
gboolean e_fclose_remove(EFILE *stream);
gboolean e_fseek(EFILE *stream, long int offset, int whence);
gboolean e_fread(void *data, size_t size, EFILE *stream);
gboolean e_fwrite(void *data, size_t size, EFILE *stream);
gboolean e_fwrite0(size_t size, EFILE *stream);
long int e_ftell(EFILE *stream);
long int e_readline(gchar **line, size_t *size, EFILE *stream);
gboolean e_copydata(EFILE *from, EFILE *to, guint32 bytes);
gboolean errdlg_copyfile(gchar *from, gchar *to);

gboolean e_fread_bswap(void *data, size_t size, EFILE *stream);
gboolean e_fwrite_bswap(void *data, size_t size, EFILE *stream);

gboolean is_same_file(char *filename1, char *filename2);
gint32 errdlg_filesize(gchar *filename);

gboolean file_exists(char *filename);
gboolean file_is_normal(char *filename);

gchar *get_filename(gboolean savemode, gchar *current_name, gchar *filemask);

#if (G_BYTE_ORDER == G_BIG_ENDIAN)
#define e_fwrite_le(a,b,c) e_fwrite_bswap(a,b,c)
#define e_fread_le(a,b,c) e_fread_bswap(a,b,c)
#else
#define e_fwrite_le(a,b,c) e_fwrite(a,b,c)
#define e_fread_le(a,b,c) e_fread(a,b,c)
#endif

#endif
