/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <pwd.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <errno.h>
#include <gtk/gtk.h>
#include "processdialog.h"
#include "mainwindow.h"
#include "datasource.h"
#include "player.h"
#include "chunk.h"
#include "sound.h"
#include "main.h"
#include "inifile.h"
#include "um.h"

#ifdef HAVE_SCHED_H
#include <sched.h>
#endif

GdkPixmap *icon = NULL;
gboolean idle_work_flag;
gboolean quitflag;

void mainloop(void)
{
     static guint player_count=0;
     if (player_work()) { player_count=0; return; }
     player_count++;
     if (gtk_events_pending()) {
	  gtk_main_iteration();
	  return;
     }
     idle_work_flag = TRUE;
     if (processdialog_active()) return;
     mainwindow_update_cursors();
     if (mainwindow_update_caches()) return;
     if (player_count > 10)
	  do_yield(TRUE);
}

int main(int argc, char **argv)
{
     int i;
     int wavefile_count=0;

     /* Check for --version and --help options. */
     for (i=1; i<argc; i++) {
	  if (!strcmp(argv[i],"--version")) { 
	       puts(PROGRAM_VERSION_STRING); 
	       return 0; 
	  } else if (!strcmp(argv[i],"--help")) {
	       printf("Syntax: %s [files]\n",argv[0]);
	       return 0;
	  } else if (!strcmp(argv[i],"--test")) {
		puts("Testing conversion functions:");
		datasource_conversion_selftest();
		puts("Testing conversion functions finished.");
		return 0;
	  } 
     }

     /* Thread init. */
#ifdef THREADSAFE
     if (!g_thread_supported()) g_thread_init(NULL);
#endif

     /* Call init functions. */
     inifile_init();
     sound_init();

     mainwindow_roll_cursor=inifile_get_gboolean("rollCursor",FALSE);

     /* Setup GTK */
     gtk_init(&argc,&argv);

     /* Search command line for filenames and create windows */
     for (i=1;i<argc;i++) {
	  gtk_widget_show(mainwindow_new_with_file(argv[i]));
	  wavefile_count++;
     }
     if (wavefile_count==0) gtk_widget_show(mainwindow_new());

     /* gtk_idle_add(idle_work,NULL); */



     /* Run it! */
     while (!quitflag)
	  mainloop();



     /* Cleanup */
     player_stop();
     sound_quit();
     inifile_quit();
     g_assert ( chunk_alive_count() == 0 && datasource_count() == 0 );
     return 0;
}

/* timeval_subtract - Stolen from glibc info docs... */

     /* Subtract the `struct timeval' values X and Y,
        storing the result in RESULT.
        Return 1 if the difference is negative, otherwise 0.  */
     
int
     timeval_subtract (result, x, y)
          GTimeVal *result, *x, *y;
     {
       /* Perform the carry for the later subtraction by updating Y. */
       if (x->tv_usec < y->tv_usec) {
         int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
         y->tv_usec -= 1000000 * nsec;
         y->tv_sec += nsec;
       }
       if (x->tv_usec - y->tv_usec > 1000000) {
         int nsec = (x->tv_usec - y->tv_usec) / 1000000;
         y->tv_usec += 1000000 * nsec;
         y->tv_sec -= nsec;
       }
     
       /* Compute the time remaining to wait.
          `tv_usec' is certainly positive. */
       result->tv_sec = x->tv_sec - y->tv_sec;
       result->tv_usec = x->tv_usec - y->tv_usec;
     
       /* Return 1 if result is negative. */
       return x->tv_sec < y->tv_sec;
     }


gchar *get_home_directory(void)
{
     static gchar *homedir = NULL;
     struct passwd *p;
     if (!homedir)
	  homedir = getenv("HOME");
     if (!homedir) {
	  p = getpwuid(getuid());
	  if (p) homedir = p->pw_dir;
     }
     if (!homedir) {
	  g_warning("Could not find home directory. Using current directory as "
		    "home directory.");
	  homedir = ".";
     }
     return homedir;
}


GdkColor *color_black(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c;
     if (first_time) {
	  gdk_color_black(gdk_colormap_get_system(),&c);
	  first_time = FALSE;
     }
     return &c;
}

GdkColor *color_white(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c;
     if (first_time) {
	  gdk_color_white(gdk_colormap_get_system(),&c);
	  first_time = FALSE;
     }
     return &c;     
}

GdkColor *color_red(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c = { 0, 0xFFFF, 0, 0 };
     if (first_time) {
	  gdk_colormap_alloc_color(gdk_colormap_get_system(),&c,FALSE,TRUE);
	  first_time = FALSE;
     }
     return &c;
}

GdkColor *color_blue(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c = { 0, 0, 0, 0xFFFF };
     if (first_time) {
	  gdk_colormap_alloc_color(gdk_colormap_get_system(),&c,FALSE,TRUE);
	  first_time = FALSE;
     }
     return &c;
}

GdkColor *color_green(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c = { 0, 0, 0xFFFF, 0 };
     if (first_time) {
	  gdk_colormap_alloc_color(gdk_colormap_get_system(),&c,FALSE,TRUE);
	  first_time = FALSE;
     }
     return &c;
}

GdkColor *color_grey(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c = { 0, 0x8000, 0x8000, 0x8000 };
     if (first_time) {
	  gdk_colormap_alloc_color(gdk_colormap_get_system(),&c,FALSE,TRUE);
	  first_time = FALSE;
     }
     return &c;
}

GdkColor *color_selection(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c = { 0, 0xE800, 0xA500, 0x4000 };
     if (first_time) {
	  gdk_colormap_alloc_color(gdk_colormap_get_system(),&c,FALSE,TRUE);
	  first_time = FALSE;
     }
     return &c;
}

GdkColor *color_mark(void)
{
     static gboolean first_time = TRUE;
     static GdkColor c = { 0, 0x0f00, 0x8400, 0x6900 };
     if (first_time) {
	  gdk_colormap_alloc_color(gdk_colormap_get_system(),&c,FALSE,TRUE);
	  first_time = FALSE;
     }
     return &c;
}


GdkGC *gc_black(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_black());
     }
     return gc;
}

GdkGC *gc_red(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_red());
     }
     return gc;
}

GdkGC *gc_green(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_green());
     }
     return gc;
}

GdkGC *gc_blue(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_blue());
     }
     return gc;
}

GdkGC *gc_grey(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_grey());
     }
     return gc;
}

GdkGC *gc_selection(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_selection());
     }
     return gc;
}

GdkGC *gc_mark(GtkWidget *w)
{
     static GdkGC *gc = NULL;
     if (!gc) {
	  gc = gdk_gc_new(w->window);
	  gdk_gc_set_foreground(gc,color_mark());
     }
     return gc;
}

void do_yield(gboolean may_sleep)
{
#ifdef HAVE_SCHED_YIELD
     static int lasttime=0, yieldcount=0;
     int i;
     if (!may_sleep) { sched_yield(); return; }
     i = time(0);
     if (i != lasttime) { yieldcount=0; lasttime=i; }
     sched_yield();
     yieldcount++;
     if (yieldcount == 20) { usleep(1000); yieldcount=0; }
#else
     if (may_sleep) usleep(1000);
#endif
}

void launch_mixer(void)
{
     gchar *m,*c;
     pid_t p;
     m = inifile_get(INI_SETTING_MIXER,INI_SETTING_MIXER_DEFAULT);
     p = fork();
     if (p == -1) {
	  c = g_strdup_printf("Error launching mixer: fork: %s",strerror(errno));
	  user_message(c,UM_OK);
	  g_free(c);
	  return;
     }
     if (p != 0) return;
     if (execlp(m,m,NULL) == -1) {
	  fprintf(stderr,"mhwaveedit: execlp: %s: %s\n",m,strerror(errno));
	  _exit(1);
     }
}

gboolean free2(gpointer key, gpointer value, gpointer user_data)
{
     g_free(key);
     g_free(value);
     return TRUE;
}
