/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <string.h>
#include <sys/time.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gtkfiles.h"
#include "filetypes.h"
#include "chunkview.h"
#include "mainwindow.h"
#include "volumedialog.h"
#include "sampleratedialog.h"
#include "speeddialog.h"
#include "samplesizedialog.h"
#include "recorddialog.h"
#include "configdialog.h"
#include "combinechannelsdialog.h"
#include "gotodialog.h"
#include "chunk.h"
#include "um.h"
#include "main.h"
#include "inifile.h"
#include "sound.h"
#include "player.h"
#include "pipedialog.h"

#include "button_open.xpm"
#include "button_save.xpm"
#include "button_undo.xpm"
#include "button_cut.xpm"
#include "button_copy.xpm"
#include "button_paste.xpm"
#include "button_pasteover.xpm"
#include "button_delete.xpm"
#include "button_cursorstart.xpm"
#include "button_cursorend.xpm"
#include "button_play.xpm"
#include "button_playselection.xpm"
#include "button_stop.xpm"
#include "button_loop.xpm"
#include "button_follow.xpm"
#include "button_record.xpm"
#include "button_mixer.xpm"
#include "icon.xpm"

#undef SHOW_DEBUG_MENU

struct HistoryEntry {
     Chunk *chunk;
     guint32 selstart,selend,viewstart,viewend,cursor;
     GHashTable *marks;
};

static GList *all_windows = NULL;
static GtkObjectClass *parent_class;
static Chunk *clipboard = NULL;
gboolean mainwindow_roll_cursor = FALSE;

static void update_desc(Mainwindow *w);

/* Some stuff for handling sound playing. */

static Mainwindow *playing_window = NULL;

static void do_stop(void)
{
     Mainwindow *w;
     player_stop();
     w = playing_window;
     playing_window = NULL;
     if (w) update_desc(w);
}

static void do_play(Mainwindow *w, guint32 start, guint32 end)
{
     if (player_play(w->view->chunk,start,end,w->loopmode)) return;
     playing_window = w;
     update_desc(w);
}

static void set_sensitive(GList *l, gboolean s)
{
     while (l != NULL) {
	  gtk_widget_set_sensitive(GTK_WIDGET(l->data),s);
	  l = l->next;
     }
}

static gchar *getfname(gchar *fullname)
{
     gchar *c;
     if (!fullname) return "(untitled)";
     c = strrchr ( fullname, '/' );
     return c ? c+1 : fullname;
}

static void fix_title(Mainwindow *wnd)
{
     gchar *c;
     if (wnd->view->chunk) 
	  c = g_strdup_printf ( "mhWaveEdit: %s (%s): %d Hz, "
				"%d bit", 
				getfname ( wnd->filename ), 
				chunk_get_time(wnd->view->chunk,
					       wnd->view->chunk->length,NULL),
				wnd->view->chunk->format.samplerate,
				wnd->view->chunk->format.samplesize*8);

     else 
	  c = g_strdup(PROGRAM_VERSION_STRING);
     gtk_window_set_title ( GTK_WINDOW ( wnd ), c );
     g_free ( c );
}

static gchar *cursortime(Mainwindow *w, gchar *buffer)
{
     if (playing_window == w && !mainwindow_roll_cursor)
	  strcpy(buffer,"<running>");
     else
	  chunk_get_time(w->view->chunk,w->view->cursorpos,buffer);
     return buffer;
}

static void update_desc(Mainwindow *w)
{
     char *c,*d;
     gchar tm1[16],tm2[16],tm3[16],tm4[16],tm5[16];
     if ( !w->view->chunk ) {
	  gtk_label_set_text(w->desc, "(no file)");
	  return;
     }
     if ( w->view->selstart != w->view->selend )
	  c = g_strdup_printf("Cursor: %s. View: [ %s - %s ]. "
			      "Selection: [ %s - %s ]",
			      cursortime(w,tm5),
			      chunk_get_time(w->view->chunk,w->view->viewstart,
					     tm1),
			      chunk_get_time(w->view->chunk,w->view->viewend, 
					     tm2),
			      chunk_get_time(w->view->chunk,w->view->selstart, 
					     tm3),
			      chunk_get_time(w->view->chunk,w->view->selend, 
					     tm4));
     else
	  c = g_strdup_printf("Cursor: %s. View: [ %s - %s ]. ",
			      cursortime(w,tm5),
			      chunk_get_time(w->view->chunk,w->view->viewstart,
					     tm1),
			      chunk_get_time(w->view->chunk,w->view->viewend, 
					     tm2));
     gtk_label_get(w->desc,&d);
     if (strcmp(d,c))
	  gtk_label_set_text(w->desc,c);
     g_free(c);
}

static gboolean mainwindow_save ( Mainwindow *w, gchar *filename )
{
     Chunk *chunk;
     gboolean fmal = FALSE;
     if (!w->changed && w->filename)
	  if (user_message("The file has not changed since last save. Press OK "
			   "if you want to save it anyway?",UM_OKCANCEL) == 
	      MR_CANCEL) 
	       return FALSE;
     if (!filename) {
	  filename = get_filename ( TRUE, w->filename, "*.wav" );
	  if (!filename) return TRUE;
	  fmal = TRUE;
     }
     chunk = chunk_save_to_file ( w->view->chunk, filename );
     if (!chunk) { if (fmal) g_free(filename); return TRUE; }
     chunk_view_set_chunk ( w->view, chunk );
     w->changed = FALSE;
     w->changecount = 0;
     if (w->filename != filename) {
	  g_free(w->filename);
	  w->filename = g_strdup(filename);
	  update_desc(w);
	  fix_title(w);
     }
     if (fmal) g_free(filename);
     return FALSE;
}

static gboolean change_check ( Mainwindow *w )
{
     gint i;
     gchar *c;
     if (!w->changed) return FALSE;
     c = g_strdup_printf( "Save changes to %s?",
			  w->filename ? w->filename : "(untitled)" );
     i = user_message ( c, UM_YESNOCANCEL );
     switch (i) {
     case MR_YES:
	  return mainwindow_save ( w, w->filename );
     case MR_NO:
	  return FALSE;
     case MR_CANCEL:
	  return TRUE;
     }
     g_assert_not_reached();
     return TRUE;
}

static void history_entry_destroy(struct HistoryEntry *h)
{
     gtk_object_unref(GTK_OBJECT(h->chunk));
     g_hash_table_foreach_remove(h->marks, free2, NULL);
     g_free(h);
}

static void mainwindow_destroy(GtkObject *obj)
{
     Mainwindow *w = MAINWINDOW(obj);
     /* puts("mainwindow_shutdown!"); */
     if (w->filename) g_free(w->filename);
     w->filename = NULL;
     parent_class->destroy(obj);

     all_windows = g_list_remove ( all_windows, obj );
     if (w->history) {
	  g_slist_foreach ( w->history, (GFunc) history_entry_destroy, NULL );
	  g_slist_free ( w->history );
	  w->history = NULL;
     }
     if (!all_windows) {
	  if (clipboard) gtk_object_unref ( GTK_OBJECT(clipboard) );
	  clipboard = NULL;
	  quitflag = TRUE;
     }
}

static gint mainwindow_delete_event(GtkWidget *widget, GdkEventAny *event)
{
     Mainwindow *w = MAINWINDOW ( widget );
     GtkWidgetClass *pcw = GTK_WIDGET_CLASS(parent_class);
     if (change_check(w)) return TRUE;
     if (playing_window == w) do_stop();
     if (pcw->delete_event) return pcw->delete_event(widget,event);
     else return FALSE;
}

static void mainwindow_realize(GtkWidget *widget)
{
     if (GTK_WIDGET_CLASS(parent_class)->realize) 
	  GTK_WIDGET_CLASS(parent_class)->realize(widget);
     if (!icon) icon = gdk_pixmap_create_from_xpm_d(widget->window,NULL,NULL,
						    icon_xpm);
     gdk_window_set_icon(widget->window,NULL,icon,NULL);
}

static void mainwindow_toggle_mark(Mainwindow *w, gchar *label)
{
     guint32 u;
     u = chunk_view_get_mark(w->view,label);
     if (u == w->view->cursorpos) 
	  chunk_view_set_mark(w->view,label,CHUNK_VIEW_BAD_MARK);
     else
	  chunk_view_set_mark(w->view,label,w->view->cursorpos);
}

static void mainwindow_goto_mark(Mainwindow *w, gchar *label)
{
     guint32 u;
     u = chunk_view_get_mark(w->view,label);
     if (u != CHUNK_VIEW_BAD_MARK) {
	  chunk_view_set_cursor(w->view, u);
	  if (w == playing_window)
	       player_set_buffer_pos(u);
     }
}

static void view_zoomin(GtkMenuItem *menu_item, gpointer user_data);
static void view_zoomout(GtkMenuItem *menu_item, gpointer user_data);
static void view_zoomtoselection(GtkMenuItem *menu_item, gpointer user_data);
static void view_zoomall(GtkMenuItem *menu_item, gpointer user_data);
static void edit_play(GtkMenuItem *menu_item, gpointer user_data);
static void edit_stop(GtkMenuItem *menu_item, gpointer user_data);

static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
{
     Mainwindow *w = MAINWINDOW(widget);
     if ((event->state & GDK_CONTROL_MASK))
	  switch (event->keyval) {
	  case GDK_0: mainwindow_toggle_mark(w,"0"); return TRUE;
	  case GDK_1: mainwindow_toggle_mark(w,"1"); return TRUE;
	  case GDK_2: mainwindow_toggle_mark(w,"2"); return TRUE;
	  case GDK_3: mainwindow_toggle_mark(w,"3"); return TRUE;
	  case GDK_4: mainwindow_toggle_mark(w,"4"); return TRUE;
	  case GDK_5: mainwindow_toggle_mark(w,"5"); return TRUE;
	  case GDK_6: mainwindow_toggle_mark(w,"6"); return TRUE;
	  case GDK_7: mainwindow_toggle_mark(w,"7"); return TRUE;
	  case GDK_8: mainwindow_toggle_mark(w,"8"); return TRUE;
	  case GDK_9: mainwindow_toggle_mark(w,"9"); return TRUE;
	  }
     else 
	  switch (event->keyval) {
	  case GDK_0: mainwindow_goto_mark(w,"0"); return TRUE;
	  case GDK_1: mainwindow_goto_mark(w,"1"); return TRUE;
	  case GDK_2: mainwindow_goto_mark(w,"2"); return TRUE;
	  case GDK_3: mainwindow_goto_mark(w,"3"); return TRUE;
	  case GDK_4: mainwindow_goto_mark(w,"4"); return TRUE;
	  case GDK_5: mainwindow_goto_mark(w,"5"); return TRUE;
	  case GDK_6: mainwindow_goto_mark(w,"6"); return TRUE;
	  case GDK_7: mainwindow_goto_mark(w,"7"); return TRUE;
	  case GDK_8: mainwindow_goto_mark(w,"8"); return TRUE;
	  case GDK_9: mainwindow_goto_mark(w,"9"); return TRUE;
	  case GDK_plus: view_zoomin(NULL,widget); return TRUE;
	  case GDK_minus: view_zoomout(NULL,widget); return TRUE;
	  case GDK_greater: view_zoomtoselection(NULL,widget); return TRUE;
	  case GDK_less: view_zoomall(NULL,widget); return TRUE;
	  case GDK_comma: edit_play(NULL,widget); return TRUE;
	  case GDK_period: edit_stop(NULL,widget); return TRUE;
	  }
     return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget,event);
}

static void mainwindow_class_init(GtkObjectClass *klass)
{
     parent_class = gtk_type_class(gtk_window_get_type());
     klass->destroy = mainwindow_destroy;
     GTK_WIDGET_CLASS(klass)->delete_event = mainwindow_delete_event;
     GTK_WIDGET_CLASS(klass)->realize = mainwindow_realize;
     GTK_WIDGET_CLASS(klass)->key_press_event = mainwindow_keypress;
}

static void mainwindow_set_chunk(Mainwindow *w, Chunk *c, gchar *filename)
{
     if (w->view->chunk) {
	  w = MAINWINDOW ( mainwindow_new() );
	  gtk_widget_show ( GTK_WIDGET ( w ) );
     }
     chunk_view_set_chunk ( w->view, c );
     if ( filename ) w->filename = g_strdup(filename);
     else w->changed = TRUE;
     fix_title(w);
     update_desc(w);
     set_sensitive(w->need_chunk_items,TRUE);
}

static void file_open(GtkMenuItem *menuitem, gpointer user_data)
{
     gchar *c;
     Chunk *chunk;
     Mainwindow *w = MAINWINDOW ( user_data );
     c = get_filename ( FALSE, w->filename, "*.wav" );
     if (!c) return;
     chunk = chunk_new_from_file ( c );
     if (!chunk) return;
     mainwindow_set_chunk ( w, chunk, c );
}

static void file_save(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     mainwindow_save ( w, w->filename );
}

static void file_saveas(GtkMenuItem *menuitem, gpointer user_data)
{
     mainwindow_save ( MAINWINDOW(user_data), NULL );
}

static void file_saveselection(GtkMenuItem *menuitem, gpointer user_data)
{
     gchar *fn;
     Chunk *c,*d;
     Mainwindow *w = MAINWINDOW(user_data);
     fn = get_filename(TRUE,NULL,"*.wav");
     if (!fn) return;
     c = chunk_get_part(w->view->chunk,w->view->selstart,
			w->view->selend-w->view->selstart);
     d = chunk_save(c,fn);
     gtk_object_sink(GTK_OBJECT(c));     
     gtk_object_sink(GTK_OBJECT(d));
}


static void file_close(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     if (change_check(w)) return;
     if (all_windows->next == NULL && w->view->chunk != NULL) {
	  do_stop();
	  chunk_view_set_chunk(w->view, NULL);
	  chunk_view_clear_marks(w->view);
	  g_slist_foreach ( w->history, (GFunc) history_entry_destroy, NULL );
	  g_slist_free ( w->history );
	  w->history = NULL;
	  g_free(w->filename);
	  w->filename = NULL;
	  w->changed = FALSE;
	  fix_title(w);
	  set_sensitive(w->need_chunk_items,FALSE);
	  set_sensitive(w->need_history_items,FALSE);
     } else
	  gtk_widget_destroy(GTK_WIDGET(w));
}

static void try_close(Mainwindow *w, gboolean *user_data)
{
     if (*user_data) return;
     *user_data = change_check(w);
}

static void file_exit(void)
{
     gboolean b = FALSE;
     g_list_foreach(all_windows,(GFunc)try_close,&b);
     if (b) return;
     do_stop();
     while (all_windows) gtk_widget_destroy(GTK_WIDGET(all_windows->data));
}

static void edit_undo(GtkMenuItem *menuitem, gpointer user_data)
{
     GSList *l;
     struct HistoryEntry *he;
     GHashTable *ht;
     Mainwindow *w = MAINWINDOW (user_data);
     g_assert ( w->history != NULL );
     do_stop();
     he = (struct HistoryEntry *)w->history->data;
     chunk_view_set_chunk ( w->view, he->chunk );
     chunk_view_set_cursor( w->view, he->cursor );
     chunk_view_set_selection( w->view, he->selstart, he->selend );
     chunk_view_set_view( w->view, he->viewstart, he->viewend );
     ht = w->view->marks;
     w->view->marks = he->marks;
     he->marks = ht;
     history_entry_destroy(he);
     l = w->history->next;
     g_slist_free_1(w->history);
     w->history = l;
     if (w->history==NULL)
	  set_sensitive(w->need_history_items, FALSE);
     w->changed = TRUE;
     if (w->changecount > 0) w->changecount --;
}

static void dup_add(gpointer key, gpointer value, gpointer user_data)
{
     gchar *new_key;
     guint32 *new_value;
     new_key = g_strdup(key);
     new_value = g_memdup(value,sizeof(guint32));
     g_hash_table_insert((GHashTable *)user_data, new_key, new_value);
}

static struct HistoryEntry *mainwindow_history_log(Mainwindow *wnd)
{
     struct HistoryEntry *he;
     he = g_malloc(sizeof(struct HistoryEntry));
     he->chunk = wnd->view->chunk;
     gtk_object_ref(GTK_OBJECT(he->chunk));
     he->cursor = wnd->view->cursorpos;
     he->selstart = wnd->view->selstart;
     he->selend = wnd->view->selend;
     he->viewstart = wnd->view->viewstart;
     he->viewend = wnd->view->viewend;
     he->marks = g_hash_table_new(g_str_hash, g_str_equal);
     g_hash_table_foreach(wnd->view->marks, dup_add, he->marks);
     return he;
}

/* FIXME: Should take a mapper function as argument instead... */

static void mainwindow_chunk_change(Mainwindow *wnd, Chunk *new_chunk, 
				    struct HistoryEntry *he, 
				    guint32 changestart, guint32 changelen,
				    gint32 increase)
{
     gint32 u,w,c;
     if (!changestart && !changelen && !increase) {
	  /* Vi ltsas som att det inte skett nn frndring p den frsta biten. */
	  changestart = MIN(wnd->view->chunk->length,new_chunk->length);
	  changelen = wnd->view->chunk->length - changestart;
	  increase = new_chunk->length - wnd->view->chunk->length;
     }
     if (!new_chunk) return;
     if (playing_window == wnd && 
	 !chunk_format_equal(new_chunk,playing_window->view->chunk)) 
	  do_stop();
     
     if (!he) he=mainwindow_history_log(wnd);
     chunk_view_set_chunk_keep_view ( wnd->view, new_chunk );
     wnd->history = g_slist_prepend ( wnd->history, he );
     set_sensitive(wnd->need_history_items, TRUE);
     wnd->changed = TRUE;
     wnd->changecount ++;
     if (increase>0) 
	  chunk_view_update_marks(wnd->view,changestart+changelen,increase);
     else if (increase<0)
	  chunk_view_update_marks(wnd->view,changestart,increase);
     if (playing_window == wnd && player_playing()) {
	  player_get_range(&u,&w);
	  c = player_get_buffer_pos();
	  /* printf("mainwindow_chunk_change: fre: u=%d,w=%d,c=%d\n",u,w,c);
	     printf("mainwindow_chunk_change: increase=%d\n",increase); */
	  if (u >= changestart+changelen) u+=increase;
	  else if (u >= changestart) u=changestart+changelen+increase;
	  if (w >= changestart+changelen) w+=increase;
	  else if (w >= changestart) w=changestart+increase;
	  if (c >= changestart+changelen) c+=increase;
	  else if (c >= changestart) c=changestart+changelen+increase;
	  if (w > new_chunk->length) w = new_chunk->length;
	  if (u < 0) u=0;
	  if (c < 0) c=0;
	  /* printf("mainwindow_chunk_change: efter: u=%d,w=%d,c=%d\n",u,w,c); */
	  if (u == w || c >= w) do_stop();
	  else player_replace(new_chunk,u,w,c);
     }
     fix_title(wnd);
     update_desc(wnd);
}

static void update_clipboard_items(Mainwindow *w, gpointer user_data)
{
     set_sensitive(w->need_clipboard_items,(int)user_data);
}

static void edit_cut(GtkMenuItem *menuitem, gpointer user_data)
{
     Chunk *chunk, *part;
     Mainwindow *w = MAINWINDOW(user_data);
     g_assert (w->view->selend != w->view->selstart);
     if (clipboard) gtk_object_unref(GTK_OBJECT(clipboard));
     part = chunk_get_part( w->view->chunk, w->view->selstart,
			    w->view->selend - w->view->selstart);
     chunk = chunk_remove_part( w->view->chunk, w->view->selstart,
				w->view->selend - w->view->selstart);
     clipboard = part;
     gtk_object_ref(GTK_OBJECT(clipboard));
     gtk_object_sink(GTK_OBJECT(clipboard));

     mainwindow_chunk_change ( w, chunk, NULL, w->view->selstart, 
			       clipboard->length, -(clipboard->length) );
     g_list_foreach(all_windows, (GFunc)update_clipboard_items, (gpointer)1);
}

static void edit_crop(GtkMenuItem *menu_item, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     Chunk *c;
     c = chunk_get_part( w->view->chunk, w->view->selstart,
			 w->view->selend - w->view->selstart);
     if (player_get_buffer_pos() < w->view->selstart)
	  do_stop();
     mainwindow_chunk_change(w, c, NULL, 0, 0, -(w->view->selstart));
     chunk_view_set_selection(w->view,0,c->length);
}

static void edit_copy(GtkMenuItem *menu_item, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     g_assert (w->view->selend != w->view->selstart);
     if (clipboard) gtk_object_unref(GTK_OBJECT(clipboard));
     clipboard = chunk_get_part(w->view->chunk, w->view->selstart, 
				w->view->selend - w->view->selstart);
     gtk_object_ref(GTK_OBJECT(clipboard));
     gtk_object_sink(GTK_OBJECT(clipboard));
     
     g_list_foreach(all_windows, (GFunc)update_clipboard_items, (gpointer)1);
}

static void edit_paste(GtkMenuItem *menu_item, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     if (!w->view->chunk) 
	  mainwindow_set_chunk(w,clipboard,NULL);
     else if (!chunk_format_equal(w->view->chunk,clipboard))
	  user_message("Can't paste: the file and the pasted data are of "
		       "different formats!",UM_OK);
     else {
	  mainwindow_chunk_change(w,chunk_insert(w->view->chunk,clipboard,
						 w->view->cursorpos), NULL,
				  w->view->cursorpos,clipboard->length,
				  +clipboard->length);
	  chunk_view_set_selection( w->view, w->view->cursorpos,
				    w->view->cursorpos + clipboard->length );
     }
}

static void edit_pasteover(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     Chunk *c;
     guint32 orig_len;
     if (!w->view->chunk) 
	  mainwindow_set_chunk(w,clipboard,NULL);
     else if (!chunk_format_equal(w->view->chunk,clipboard))
	  user_message("Can't paste: the file and the pasted data are of "
		       "different formats!",UM_OK);
     else {
	  orig_len = MIN(w->view->chunk->length-w->view->cursorpos,
			 clipboard->length);
	  c = chunk_replace_part(w->view->chunk, w->view->cursorpos, 
				 orig_len, clipboard);
	  mainwindow_chunk_change(w,c,NULL,w->view->cursorpos,orig_len,
				  clipboard->length-orig_len);
	  chunk_view_set_selection( w->view, w->view->cursorpos,
				    w->view->cursorpos + clipboard->length );
     }     
}

static void edit_mixpaste(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     Chunk *c,*p,*y;
     guint32 partlen;
     if (!w->view->chunk) 
	  mainwindow_set_chunk(w,clipboard,NULL);
     else if (!chunk_format_equal(w->view->chunk,clipboard))
	  user_message("Can't paste: the file and the pasted data are of "
		       "different formats!",UM_OK);
     else {
	  p = chunk_get_part(w->view->chunk,w->view->cursorpos,clipboard->length);
	  partlen = p->length;
	  y = chunk_mix(p,clipboard);
	  gtk_object_sink(GTK_OBJECT(p));
	  if (!y) return;
	  c = chunk_replace_part(w->view->chunk,w->view->cursorpos,partlen,y);
	  gtk_object_sink(GTK_OBJECT(y));
	  mainwindow_chunk_change(w,c,NULL,w->view->cursorpos,
				  partlen,clipboard->length-partlen);
	  chunk_view_set_selection( w->view, w->view->cursorpos,
				    w->view->cursorpos + clipboard->length );
     }     
}

static void edit_pastetonew(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW (user_data);
     mainwindow_set_chunk(w,clipboard,NULL);
}

static void edit_delete(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     Chunk *chunk;
     gint32 slen;
     slen = w->view->selend - w->view->selstart;
     chunk = chunk_remove_part(w->view->chunk, w->view->selstart, slen);
     mainwindow_chunk_change ( w, chunk, NULL , w->view->selstart, slen, -slen);
}

static void edit_selectall(GtkMenuItem *menuitem, gpointer user_data)
{
     chunk_view_set_selection(MAINWINDOW(user_data)->view,0,~0);
}

static void view_zoomin(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     chunk_view_zoom(w->view,2.0);
}

static void view_zoomout(GtkMenuItem *item, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     chunk_view_zoom(w->view,0.5);
}

static void view_zoomtoselection(GtkMenuItem *menuitem, gpointer user_data)
{
     ChunkView *cv;
     cv = MAINWINDOW(user_data)->view;
     chunk_view_set_view(cv,cv->selstart,cv->selend);
}

static void view_zoomall(GtkMenuItem *menuitem, gpointer user_data)
{
     ChunkView *cv;
     cv = MAINWINDOW(user_data)->view;
     chunk_view_set_view(cv,0,~0);
}

#ifdef SHOW_DEBUG_MENU
static void debug_mark(GtkMenuItem *menuitem, gpointer user_data)
{
     MAINWINDOW(user_data)->changed = TRUE;
}

static gboolean dummy_proc(void *in, guint sample_size, 
			   chunk_writeout_func out_func, Chunk *source)
{
     return out_func(in,sample_size);
}

static void debug_dummy(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     mainwindow_chunk_change(w,chunk_filter(w->view->chunk,dummy_proc,NULL,FALSE,FALSE),NULL);
}

static void checkoc_proc(Chunk *chunk, gpointer user_data)
{
     if (chunk->opencount > 0) 
	  printf("Chunk %p has opencount=%d\n",chunk,chunk->opencount);
}

static void debug_checkoc(GtkMenuItem *menuitem, gpointer user_data)
{
     puts("--------");
     chunk_foreach((GFunc)checkoc_proc,NULL);
}
#endif

static void effects_volume(GtkMenuItem *menuitem, gpointer user_data)
{
     gtk_widget_show(volume_dialog_new(MAINWINDOW(user_data)));
}

static void effects_speed(GtkMenuItem *menuitem, gpointer user_data)
{
     gtk_widget_show(speed_dialog_new(MAINWINDOW(user_data)));
}

static void effects_samplerate(GtkMenuItem *menuitem, gpointer user_data)
{    
     gtk_widget_show(samplerate_dialog_new(MAINWINDOW(user_data)));
}

static void effects_samplesize(GtkMenuItem *menuitem, gpointer user_data)
{
     gtk_widget_show(samplesize_dialog_new(MAINWINDOW(user_data)));
}

static void effects_mixchannels(GtkMenuItem *menuitem, gpointer user_data)
{
     Chunk *c;
     Mainwindow *w = MAINWINDOW(user_data);
     do_stop();
     if (w->view->chunk->format.channels == 1)
	  user_message("There already is only one channel!",UM_OK);
     else {
	  c = chunk_onechannel(w->view->chunk);
	  if (c) mainwindow_chunk_change(w,c,NULL,0,0,0);
     }
}

static void effects_splitchannel(GtkMenuItem *menuitem, gboolean user_data)
{
     Mainwindow *w = MAINWINDOW (user_data);
     Chunk *c;
     c = chunk_copy_channel(w->view->chunk,0);
     if (c == NULL) return;
     do_stop();
     mainwindow_chunk_change(w,c,NULL,0,0,0);
}

static void edit_stop(GtkMenuItem *menu_item, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     if (!playing_window) {
	  if (w->view->cursorpos != 0 || w->view->chunk->length == 0)
	       chunk_view_set_cursor(w->view,0);
	  else
	       chunk_view_set_cursor(w->view,w->view->chunk->length-1);
     } else
	  do_stop();
}


gboolean mainwindow_update_cursors(void)
{
     guint32 ui=0;
     guint32 s,e;
     Mainwindow *w;

     if (!playing_window) return FALSE;

     if (!player_playing()) {
	  player_get_range(&s,&e);
	  chunk_view_set_cursor(playing_window->view, s);
	  w = playing_window;
	  playing_window = NULL;
	  update_desc(w);
	  return FALSE;
     }

     ui = player_get_real_pos();
     if (ui != playing_window->view->cursorpos)
	  chunk_view_set_cursor(playing_window->view, ui);

     return TRUE;
}

gboolean mainwindow_update_caches(void)
{
     static Mainwindow *window = NULL;
     GList *l;
     l = all_windows;
     if (window != NULL) {
	  while (l != NULL && l->data != window) l=l->next;
	  if (l == NULL) {
	       window = NULL;
	       l = all_windows;
	  }
     }
     while (1) {
	  if (chunk_view_update_cache(MAINWINDOW(l->data)->view)) {
	       window = l->data;
	       return TRUE;
	  }	  
	  l=l->next;
	  if (l==NULL) {
	       if (window == NULL) return FALSE;
	       l=all_windows;
	       window = NULL;
	  }	  
     }
}

static void edit_playselection(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW (user_data);
     if (w->view->selstart != w->view->selend)
	  do_play(w,w->view->selstart,w->view->selend);
     else
	  do_play(w,0,w->view->chunk->length);
}

static void edit_playall(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW (user_data);
     do_play(w,0,w->view->chunk->length);
}

static void help_about(void)
{
     gchar *p;
     GtkWidget *a,*b,*c;
     a = gtk_window_new(GTK_WINDOW_DIALOG);
     gtk_window_set_title(GTK_WINDOW(a),"About mhWaveEdit");
     gtk_container_set_border_width(GTK_CONTAINER(a),5);
     b = gtk_vbox_new(FALSE,10);
     gtk_container_add(GTK_CONTAINER(a),b);
     gtk_widget_show(b);
     c = gtk_label_new(PROGRAM_VERSION_STRING);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_label_new("Created by Magnus Hjorth (magnus.hjorth@home.se)\n"
	  "Copyright 2002-2003, Magnus Hjorth");
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);     
     p = g_strjoin(NULL,"Current sound driver: ",sound_driver_name(),NULL);
     c = gtk_label_new(p);
     g_free(p);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_label_new("Compiled " __DATE__ " " __TIME__ );
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
#ifdef USE_DOUBLE_SAMPLES
     c = gtk_label_new("Uses double-precision math");
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
#endif
     c = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_label_new("Distributed under GNU General Public License.\n"
		       "For information, see the file COPYING");
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     c = gtk_button_new_with_label("OK");
     gtk_signal_connect_object(GTK_OBJECT(c),"clicked",
			       (GtkSignalFunc)gtk_widget_destroy,GTK_OBJECT(a));
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show(c);
     
     gtk_widget_show(a);
}

static void edit_play(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW (user_data);
     do_play(w,w->view->cursorpos,w->view->chunk->length);
}

static void edit_selstartcursor(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     if (w->view->selstart == w->view->selend) 
	  chunk_view_set_selection(w->view,w->view->cursorpos,~0);
     else 
	  chunk_view_set_selection(w->view,w->view->cursorpos,w->view->selend);
}

static void edit_selendcursor(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     if (w->view->selstart == w->view->selend)
	  chunk_view_set_selection(w->view,0,w->view->cursorpos+1);
     else
	  chunk_view_set_selection(w->view,w->view->selstart,
				   w->view->cursorpos+1);
}

static void edit_record(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *wnd = MAINWINDOW (user_data);
     RecordDialog *rd;
     Chunk *c;
     do_stop();
     rd = RECORD_DIALOG(record_dialog_new());
     c = record_dialog_execute(rd);
     if (c) mainwindow_set_chunk(wnd,c,NULL);
}

static void edit_preferences(GtkMenuItem *menuitem, gpointer user_data)
{
    gtk_widget_show(config_dialog_new());
}

static void effects_combinechannels(GtkMenuItem *menuitem, gpointer user_data)
{
    Mainwindow *w = MAINWINDOW ( user_data );
    gtk_widget_show(combine_channels_dialog_new(w));
}

static void view_timescale(GtkCheckMenuItem *checkmenuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     chunk_view_set_timescale(w->view, checkmenuitem->active);
}

static void edit_insertsilence(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     gfloat f;
     Chunk *c;
     if (user_input_float("Seconds of silence: ","Insert Silence",0.0,&f)) 
	  return;
     if (f<=0.0) return;
     c = chunk_new_silent(&(w->view->chunk->format),f);
     mainwindow_chunk_change(w,chunk_insert(w->view->chunk,c,
					    w->view->cursorpos),NULL,
			     w->view->cursorpos,0,c->length);
     chunk_view_set_selection( w->view, w->view->cursorpos,
			       w->view->cursorpos + c->length );
     gtk_object_sink(GTK_OBJECT(c));
}

static void edit_clearundobuffer(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     guint l;
     GSList *sl,*pr=NULL;
     l = g_slist_length(w->history);
     g_assert(l >= w->changecount);
     if (l == w->changecount || w->changecount == 0) {
	  if (user_message("This will clear the entire undo buffer!",
			   UM_OKCANCEL) == MR_CANCEL)
	       return;
	  sl = w->history;
	  g_slist_foreach(sl,(GFunc)history_entry_destroy,NULL);
	  w->history = NULL;
	  set_sensitive(w->need_history_items,FALSE);
     } else {
	  if (user_message("This will clear the undo buffer up to the last "
			   "save",UM_OKCANCEL) == MR_CANCEL)
	       return;
	  pr = g_slist_nth(w->history, w->changecount-1);
	  sl = pr->next;
	  g_slist_foreach(sl,(GFunc)history_entry_destroy,NULL);
	  pr->next = NULL;
     }
     g_slist_free(sl);     
     w->changecount = 0;
}

static void edit_clearclipboard(GtkMenuItem *menuitem, gpointer user_data)
{
     gtk_object_unref(GTK_OBJECT(clipboard));
     clipboard = NULL;
     g_list_foreach(all_windows, (GFunc)update_clipboard_items, (gpointer)0);
}

static void edit_positioncursor(GtkMenuItem *menuitem, gpointer user_data)
{
     gtk_widget_show(goto_dialog_new(MAINWINDOW(user_data)));
}

static void effects_pipe(GtkMenuItem *menuitem, gpointer user_data)
{
     gtk_widget_show(pipe_dialog_new(MAINWINDOW(user_data)));
}

static void cursor_tobeginning(GtkMenuItem *menuitem, gpointer user_data)
{
     chunk_view_set_cursor(MAINWINDOW(user_data)->view,0);
}

static void cursor_toend(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     chunk_view_set_cursor(w->view,w->view->chunk->length-1);
}

static void cursor_toselstart(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     chunk_view_set_cursor(w->view,w->view->selstart);
}

static void cursor_toselend(GtkMenuItem *menuitem, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     chunk_view_set_cursor(w->view,w->view->selend-1);
}

static GtkWidget *create_menu(Mainwindow *w)
{
     guint i;
     GtkWidget *item;

     GtkItemFactoryEntry menu_items[] = {
	  { "/_File",         NULL,         NULL,           0, "<Branch>"    },
	  { "/File/_Open...", "<control>O", file_open,      0, NULL          },
	  { "/File/_Save",    "<control>S", file_save,      0, NULL          },
	  { "/File/Save _as...",NULL,       file_saveas,    0, NULL          },
	  { "/File/Save selection as...",NULL,file_saveselection,0,NULL      },
	  { "/File/sep1",     NULL,         NULL,           0, "<Separator>" },
	  { "/File/_Close",   NULL,         file_close,     0, NULL          },
	  { "/File/sep2",     NULL,         NULL,           0, "<Separator>" },
	  { "/File/_Exit",    NULL,         file_exit,      0, NULL          },
	  { "/_Edit",         NULL,         NULL,           0, "<Branch>"    },
	  { "/Edit/_Undo",    "<control>Z", edit_undo,      0, NULL          },
	  { "/Edit/sep1",     NULL,         NULL,           0, "<Separator>" },
	  { "/Edit/Cu_t",     "<control>X", edit_cut,       0, NULL          },
	  { "/Edit/_Copy",    "<control>C", edit_copy,      0, NULL          },
	  { "/Edit/_Paste",   "<control>P", edit_paste,     0, NULL          },
	  { "/Edit/Paste _over",NULL,       edit_pasteover, 0, NULL          },
	  { "/Edit/_Mix paste",NULL,        edit_mixpaste,  0, NULL          },
	  { "/Edit/Insert _silence",NULL,   edit_insertsilence,0,NULL        },
	  { "/Edit/Paste to _new",NULL,     edit_pastetonew,0, NULL          },
	  { "/Edit/Cr_op",    NULL,         edit_crop,      0, NULL          },
	  { "/Edit/_Delete",  "<control>D", edit_delete,    0, NULL          },
	  { "/Edit/sep2",     NULL,         NULL,           0, "<Separator>" },
	  { "/Edit/Select _all","<control>A",edit_selectall,0, NULL          },
	  { "/Edit/sep3",     NULL,         NULL,           0, "<Separator>" },
	  { "/Edit/Clear clipboard",NULL,   edit_clearclipboard,0,NULL       },
	  { "/Edit/Clear undo buffer",NULL, edit_clearundobuffer,0,NULL      },
	  { "/Edit/sep4",     NULL,         NULL,           0, "<Separator>" },
	  { "/Edit/P_references",NULL,      edit_preferences,0,NULL          },
	  { "/_View",         NULL,         NULL,           0, "<Branch>"    },
	  { "/View/Zoom _in", NULL,         view_zoomin,    0, NULL          },
	  { "/View/Zoom _out",NULL,         view_zoomout,   0, NULL          },
	  { "/View/Zoom to _selection",NULL,view_zoomtoselection,0,NULL      },
	  { "/View/sep1",     NULL,         NULL,           0, "<Separator>" },
	  { "/View/Zoom _all",NULL,         view_zoomall,   0, NULL          },
	  { "/View/sep2",     NULL,         NULL,           0, "<Separator>" },
	  { "/View/_Time scale",NULL,       view_timescale, 0, "<CheckItem>" },
	  { "/_Cursor",       NULL,         NULL,           0, "<Branch>" },
	  {"/Cursor/Set selection start","<control>Q",edit_selstartcursor,0,
	   NULL},
	  { "/Cursor/Set selection end","<control>W",edit_selendcursor,0,NULL},
	  { "/Cursor/sep1",   NULL,         NULL,           0, "<Separator>" },
	  {"/Cursor/Move to beginning","<control>H",cursor_tobeginning,0,
	   NULL},
	  {"/Cursor/Move to end","<control>J",cursor_toend, 0, NULL          },
	  {"/Cursor/Move to selection start","<control>K",cursor_toselstart,
	   0,NULL},
	  {"/Cursor/Move to selection end","<control>L",cursor_toselend,0,
	   NULL},
	  { "/Cursor/sep2",   NULL,         NULL,           0, "<Separator>" },
	  {"/Cursor/Position cursor...","<control>G",edit_positioncursor,0,
	   NULL},
	  { "/_Play",         NULL,         NULL,           0, "<Branch>"    },
	  { "/Play/_Play from cursor",NULL, edit_play,      0, NULL          },
	  { "/Play/Play _all",NULL,         edit_playall,   0, NULL          },
	  { "/Play/Play se_lection",NULL,   edit_playselection,0,NULL        },
	  { "/Play/_Stop",    NULL,         edit_stop,      0, NULL          },
	  { "/Play/sep1",     NULL,         NULL,           0, "<Separator>" },
	  { "/Play/_Record",  NULL,         edit_record,    0, NULL          },
	  { "/Effec_ts",      NULL,         NULL,           0, "<Branch>"    },
	  { "/Effects/_Volume adjust (fade)...",NULL,effects_volume,0,NULL   },
	  { "/Effects/_Speed...",NULL,      effects_speed,  0, NULL          },
	  { "/Effects/sep1",  NULL,         NULL,           0, "<Separator>" },
	  { "/Effects/Convert sample_rate...",NULL,effects_samplerate,0,NULL },
	  { "/Effects/Convert samples_ize...",NULL,effects_samplesize,0,NULL },
	  { "/Effects/sep2",  NULL,         NULL,           0, "<Separator>" },
	  { "/Effects/_Mix channels...",NULL,effects_mixchannels,0,NULL      },
	  { "/Effects/Sp_lit channel...",NULL,effects_splitchannel,0,NULL    },
	  {"/Effects/_Combine channels...",NULL,effects_combinechannels,0,NULL},
	  { "/Effects/sep3",  NULL,         NULL,           0, "<Separator>" },
	  { "/Effects/Pipe through program...",NULL,effects_pipe,0,NULL      },
#ifdef SHOW_DEBUG_MENU
	  { "/Debug",         NULL,         NULL,           0, "<Branch>"    },
	  { "/Debug/Mark as modified",NULL, debug_mark,     0, NULL          },
	  { "/Debug/Dummy effect",NULL,     debug_dummy,    0, NULL          },
	  { "/Debug/Check opencount",NULL,  debug_checkoc,  0, NULL          },
#endif
	  { "/_Help",         NULL,         NULL,           0, "<LastBranch>"},
	  { "/Help/_About",   NULL,         help_about,     0, NULL          },
     };

     gchar *need_chunk_names[] = {
	  "/File/Save", "/File/Save as...", 
	  "/Edit/Insert silence", 
	  "/Edit/Select all", "/Cursor", "/View", "/Play/Play from cursor",
	  "/Play/Play all", "/Play/Stop", "/Effects"
     };

     gchar *need_selection_names[] = {
	  "/Edit/Cut", "/Edit/Copy", "/Edit/Delete", "/Edit/Crop", 
	  "/View/Zoom to selection", "/File/Save selection as...",
	  "/Play/Play selection", "/Cursor/Move to selection start", 
	  "/Cursor/Move to selection end"
     };

     gchar *need_clipboard_names[] = {
	  "/Edit/Paste", "/Edit/Paste over", "/Edit/Mix paste", 
	  "/Edit/Paste to new", "/Edit/Clear clipboard"
     };

     gchar *need_history_names[] = {
	  "/Edit/Undo", "/Edit/Clear undo buffer"
     };
  
     GtkAccelGroup *accel_group;
     GtkItemFactory *item_factory;
     
     accel_group = gtk_accel_group_new();
     item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,"<main>",
					 accel_group);
     gtk_item_factory_create_items_ac(item_factory,ARRAY_LENGTH(menu_items),
				      menu_items,w,2);
     gtk_window_add_accel_group(GTK_WINDOW(w),accel_group);

     for (i=0; i<ARRAY_LENGTH(need_chunk_names); i++)
	  w->need_chunk_items = 
	       g_list_append(w->need_chunk_items,gtk_item_factory_get_item(
				  item_factory,need_chunk_names[i]));

     for (i=0; i<ARRAY_LENGTH(need_selection_names); i++)
	  w->need_selection_items = 
	       g_list_append(w->need_selection_items,gtk_item_factory_get_item(
				  item_factory,need_selection_names[i]));

     for (i=0; i<ARRAY_LENGTH(need_clipboard_names); i++)
	  w->need_clipboard_items = 
	       g_list_append(w->need_clipboard_items,gtk_item_factory_get_item(
				  item_factory,need_clipboard_names[i]));
     
     for (i=0; i<ARRAY_LENGTH(need_history_names); i++)
	  w->need_history_items = 
	       g_list_append(w->need_history_items,gtk_item_factory_get_item(
				  item_factory,need_history_names[i]));

     item = gtk_item_factory_get_item(item_factory,"/View/Time scale");
     gtk_check_menu_item_set_active(
	  GTK_CHECK_MENU_ITEM(item),
	  inifile_get_gboolean(INI_SETTING_TIMESCALE,
			       INI_SETTING_TIMESCALE_DEFAULT));

     return gtk_item_factory_get_widget(item_factory,"<main>");

}

static void loopmode_toggle(GtkToggleButton *button, gboolean *user_data)
{
     *user_data = gtk_toggle_button_get_active(button);
}

static void followmode_toggle(GtkToggleButton *button, gboolean *user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     chunk_view_set_followmode(w->view, gtk_toggle_button_get_active(button));
}

static GtkWidget *create_toolbar(Mainwindow *w)
{
     GtkWidget *t,*b,*r;
     GdkPixmap *p;
     GdkBitmap *bmp;
#if GTK_MAJOR_VERSION == 2
     t = gtk_toolbar_new();
#else
     t = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
#endif
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_open_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Load a file from disk","X",b,GTK_SIGNAL_FUNC(file_open),w);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_save_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Save the current file to disk","X",b,GTK_SIGNAL_FUNC(file_save),w);
     w->need_chunk_items = g_list_append(w->need_chunk_items,r);
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_undo_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Undo the last change","X",b,GTK_SIGNAL_FUNC(edit_undo),w);
     w->need_history_items = g_list_append(w->need_history_items, r);
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_cut_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Cut out the current selection","X",b,GTK_SIGNAL_FUNC(edit_cut),w);
     w->need_selection_items = g_list_append(w->need_selection_items, r);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_copy_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Copy the current selection","X",b,GTK_SIGNAL_FUNC(edit_copy),w);
     w->need_selection_items = g_list_append(w->need_selection_items, r);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_paste_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Paste at cursor position","X",b,GTK_SIGNAL_FUNC(edit_paste),w);
     w->need_clipboard_items = g_list_append(w->need_clipboard_items, r);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_pasteover_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Paste, overwriting the data after the cursor position","X",b,
	  GTK_SIGNAL_FUNC(edit_pasteover),w);
     w->need_clipboard_items = g_list_append(w->need_clipboard_items, r);
     gtk_widget_set_sensitive(r,FALSE);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_delete_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Delete the selection","X",b,GTK_SIGNAL_FUNC(edit_delete),w);
     w->need_selection_items = g_list_append(w->need_selection_items, r);
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_cursorstart_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Set selection start to cursor position","X",b,
	  GTK_SIGNAL_FUNC(edit_selstartcursor),w);
     w->need_chunk_items = g_list_append(w->need_chunk_items, r);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_cursorend_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Set selection end to cursor position","X",b,
	  GTK_SIGNAL_FUNC(edit_selendcursor),w);
     w->need_chunk_items = g_list_append(w->need_chunk_items, r);
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_play_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Play from cursor position","X",b,
	  GTK_SIGNAL_FUNC(edit_play),w);
     w->need_chunk_items = g_list_append(w->need_chunk_items, r);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_playselection_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Play selected area","X",b,
	  GTK_SIGNAL_FUNC(edit_playselection),w);
     w->need_chunk_items = g_list_append(w->need_chunk_items,r);
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_stop_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Stop playing","X",b,
	  GTK_SIGNAL_FUNC(edit_stop),w);
     w->need_chunk_items = g_list_append(w->need_chunk_items, r);
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_loop_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,NULL,
	  "Loop mode (play over and over)","X",b,
	  GTK_SIGNAL_FUNC(loopmode_toggle),&(w->loopmode));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_follow_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_TOGGLEBUTTON,NULL,NULL,
	  "Follow cursor while playing","X",b,
	  GTK_SIGNAL_FUNC(followmode_toggle),w);
     
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_record_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Record","X",b,GTK_SIGNAL_FUNC(edit_record),w);
     gtk_widget_set_sensitive(r,input_supported());
     gtk_toolbar_append_space(GTK_TOOLBAR(t));
     p = gdk_pixmap_colormap_create_from_xpm_d(
	  NULL, gtk_widget_get_colormap(GTK_WIDGET(w)), &bmp, NULL,
	  button_mixer_xpm);
     b = gtk_pixmap_new(p, bmp);
     r = gtk_toolbar_append_element(
	  GTK_TOOLBAR(t),GTK_TOOLBAR_CHILD_BUTTON,NULL,NULL,
	  "Launch mixer","X",b,GTK_SIGNAL_FUNC(launch_mixer),w);     
     return t;
}

static void mainwindow_view_changed(ChunkView *cv, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW(user_data);
     w->view_adj->page_size = (cv->viewend - cv->viewstart);
     w->view_adj->upper = cv->chunk->length;
     w->view_adj->value = cv->viewstart;
     w->view_adj->step_increment = 16;
     w->view_adj->page_increment = w->view_adj->page_size / 2;
     gtk_adjustment_changed ( w->view_adj );
     update_desc(w);
}

static void mainwindow_selection_changed(ChunkView *cv, gpointer user_data)
{
     Mainwindow *w = MAINWINDOW ( user_data );
     update_desc ( w );
     set_sensitive(w->need_selection_items,
		   (w->view->selstart != w->view->selend));
}

static void mainwindow_cursor_changed(ChunkView *cv, gboolean user, 
				      gpointer user_data)
{
     Mainwindow *w;
     w = MAINWINDOW ( user_data );
     if (w!=playing_window || mainwindow_roll_cursor) update_desc(w);
     if (!user) return;
     if (w == playing_window && player_playing()) 
	  player_set_buffer_pos(cv->cursorpos);
}

static void mainwindow_value_changed(GtkAdjustment *adjustment, gpointer user_data)
{
     chunk_view_set_view( MAINWINDOW(user_data)->view, adjustment->value, 
			  adjustment->value + adjustment->page_size );
}

static void mainwindow_init(Mainwindow *obj)
{
     GtkWidget *a,*b;
     GtkRequisition req;

     /* Stll in datastrukturen */
     obj->view = CHUNKVIEW(chunk_view_new());

     chunk_view_set_timescale(
	  obj->view, inifile_get_gboolean(INI_SETTING_TIMESCALE,
					  INI_SETTING_TIMESCALE_DEFAULT));
     gtk_signal_connect( GTK_OBJECT(obj->view), "view-changed",
			 GTK_SIGNAL_FUNC(mainwindow_view_changed), obj );
     gtk_signal_connect( GTK_OBJECT(obj->view), "selection-changed",
			 GTK_SIGNAL_FUNC(mainwindow_selection_changed), obj );
     gtk_signal_connect( GTK_OBJECT(obj->view), "cursor-changed",
			 GTK_SIGNAL_FUNC(mainwindow_cursor_changed), obj );
     obj->view_adj = GTK_ADJUSTMENT( gtk_adjustment_new ( 0,0,0,0,0,0 ));
     gtk_signal_connect( GTK_OBJECT(obj->view_adj), "value-changed", 
			 GTK_SIGNAL_FUNC(mainwindow_value_changed), obj);
     obj->desc = GTK_LABEL ( gtk_label_new ( "(no file)" ) ) ;
     gtk_misc_set_alignment ( GTK_MISC(obj->desc), 0.0, 0.5 );
     obj->filename = NULL;
     obj->changed = FALSE;
     obj->changecount = 0;
     obj->history = NULL;
     obj->loopmode = FALSE;

     obj->need_chunk_items = NULL;
     obj->need_selection_items = NULL;
     obj->need_clipboard_items = NULL;
     obj->need_history_items = NULL;

     fix_title ( obj );
     
     /* Bygg upp komponenterna */
     a = gtk_vbox_new( FALSE, 0 );
     gtk_container_add(GTK_CONTAINER(obj),a);
     gtk_widget_show(a);
     b = create_menu( obj );
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     b = create_toolbar( obj );
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);     
     gtk_container_add(GTK_CONTAINER(a),GTK_WIDGET(obj->view));
     gtk_widget_show(GTK_WIDGET(obj->view));
     b = gtk_hscrollbar_new ( obj->view_adj );
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);

     b = gtk_event_box_new();
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     
     /* Little hack for the status bar label */
     gtk_widget_size_request(GTK_WIDGET(obj->desc),&req);
     gtk_widget_set_usize(GTK_WIDGET(obj->desc),req.width,req.height);

     gtk_container_add(GTK_CONTAINER(b),GTK_WIDGET(obj->desc));
     gtk_widget_show(GTK_WIDGET(obj->desc));

     /* gtk_widget_set_usize(GTK_WIDGET(obj),450,250); */
     gtk_window_set_policy(GTK_WINDOW(obj),TRUE,TRUE,FALSE);
     
     all_windows = g_list_append ( all_windows, obj );

     set_sensitive(obj->need_chunk_items, FALSE);
     set_sensitive(obj->need_selection_items, FALSE);
     set_sensitive(obj->need_clipboard_items, clipboard != NULL);
     set_sensitive(obj->need_history_items, FALSE);
}

guint mainwindow_get_type(void)
{
static guint id=0;
if (!id) {
	GtkTypeInfo info = {
	     "Mainwindow",
	     sizeof(Mainwindow),
	     sizeof(MainwindowClass),
	     (GtkClassInitFunc) mainwindow_class_init,
	     (GtkObjectInitFunc) mainwindow_init 
	};
	id=gtk_type_unique(gtk_window_get_type(),&info);
	}
return id;
}

GtkWidget *mainwindow_new(void)
{
     return GTK_WIDGET( gtk_type_new(mainwindow_get_type()) );
}

GtkWidget *mainwindow_new_with_file(char *filename)
{
     Mainwindow *w;
     Chunk *c;
     w = MAINWINDOW ( mainwindow_new() );
     c = chunk_new_from_file(filename);
     if (c) mainwindow_set_chunk(w, c, filename);
     return GTK_WIDGET(w);
}

void mainwindow_effect(Mainwindow *w, chunk_filter_proc proc, 
		       chunk_filter_proc eof_proc, gboolean allchannels,
		       gboolean convert)
{
     Chunk *c,*p,*r;
     guint32 u,plen,rlen;
     if ((w->view->selstart == w->view->selend) ||
	 (w->view->selstart==0 && w->view->selend >= w->view->chunk->length)) {
	  r = chunk_filter( w->view->chunk, proc, eof_proc,allchannels,convert);
	  if (r) { mainwindow_chunk_change ( w, r, NULL, 0, 0, 0 ); }
     } else {
	  u = w->view->selstart;
	  p = chunk_get_part(w->view->chunk, u, 
			     w->view->selend - u);
	  plen = p->length;
	  r = chunk_filter( p, proc, eof_proc, allchannels, convert);
	  gtk_object_sink(GTK_OBJECT(p));
	  if (r) {
	       rlen = r->length;
	       c = chunk_replace_part(w->view->chunk,u,plen,r);
	       gtk_object_sink(GTK_OBJECT(r));
	       mainwindow_chunk_change( w, c, NULL, u, plen, rlen - plen);
	       chunk_view_set_selection(w->view,u,u+rlen);
	  }
     }
}

void mainwindow_effect_manual(Mainwindow *w, mainwindow_effect_proc proc, 
			      gboolean selection_only, gpointer user_data)
{
     Chunk *c,*p,*r;
     guint32 u,plen,rlen;
     if ((w->view->selstart == w->view->selend) ||
	 (w->view->selstart==0 && w->view->selend >= w->view->chunk->length) ||
	  !selection_only) {
	  r = proc( w->view->chunk, w, user_data );
	  if (r) { do_stop(); mainwindow_chunk_change ( w, r, NULL, 0, 0, 0 ); }
     } else {
	  u = w->view->selstart;
	  p = chunk_get_part(w->view->chunk, u, 
			     w->view->selend - u);
	  plen = p->length;
	  r = proc( p, w, user_data );
	  if (r) {
	       rlen = r->length;
	       c = chunk_replace_part(w->view->chunk,u,plen,r);
	       gtk_object_sink(GTK_OBJECT(r));
	       mainwindow_chunk_change( w, c, NULL, u, plen, rlen - plen);
	       chunk_view_set_selection(w->view,u,u+rlen);
	  }
     }     
}

void mainwindow_parse(Mainwindow *w, chunk_parse_proc proc, gboolean allchannels,
		      gboolean convert)
{
     Chunk *c;
     if ((w->view->selstart == w->view->selend) ||
	 (w->view->selstart==0 && w->view->selend >= w->view->chunk->length)) {
	  chunk_parse(w->view->chunk,proc,allchannels,convert);
     } else {
	  c = chunk_get_part(w->view->chunk,w->view->selstart,
			     w->view->selend - w->view->selstart);
	  chunk_parse(c,proc,allchannels,convert);
	  gtk_object_sink(GTK_OBJECT(c));
     }
}

guint32 mainwindow_parse_length(Mainwindow *w)
{
     if (w->view->selstart != w->view->selend)
	  return w->view->selend - w->view->selstart;
     else
	  return w->view->chunk->length;
}

void mainwindow_update_texts(void)
{
     g_list_foreach(all_windows,(GFunc)fix_title,NULL);
     g_list_foreach(all_windows,(GFunc)update_desc,NULL);
}
