/*
 * Copyright (C) 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


/* This module handles the sound playing. */

#ifndef PLAYER_H_INCLUDED
#define PLAYER_H_INCLUDED

#include <gtk/gtk.h>
#include "chunk.h"

/* This function must be called periodically to make sure that sound data gets
 * to the sound driver. It returns FALSE if it had nothing to do.
 */

gboolean player_work(void);

/* Play a certain chunk. Returns TRUE if anything failed.  */

gboolean player_play(Chunk *chunk, guint32 startpos, guint32 endpos, 
		     gboolean loop);

/* TRUE if the play thread is currently playing. */

gboolean player_playing(void);

/* Returns start- and endpoint of the currently playing part. */

void player_get_range(guint32 *startpos, guint32 *endpos);

/* Stops playing (or does nothing if not playing). */

void player_stop(void);

/* Gets the current position of the player thread in the data buffer. */

guint32 player_get_buffer_pos(void);

/* Gets the current position which is actually playing right now. This function
 * compensates for the buffering done by the sound drivers. 
 */

guint32 player_get_real_pos(void);

/* Use another chunk as sound source but keep all the buffered data intact. */

void player_replace(Chunk *chunk, guint32 start, guint32 end, guint32 pos);

/* Changes the buffer position. */

void player_set_buffer_pos(guint32 pos);

#endif
