/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <time.h>
#include <gtk/gtk.h>
#include "processdialog.h"
#include "main.h"

static gboolean active=FALSE;
static guint32 total,current,tresh,accum;
static time_t lasttime;
static GtkWindow *wnd;
static GtkProgressBar *bar;
static GtkLabel *lbl;
static gboolean window_showed;

static void processdialog_destroy(void)
{
     wnd = NULL;
     active = FALSE;
}

static void create_wnd(void)
{
     GtkWidget *a, *b;
     wnd = GTK_WINDOW(gtk_window_new(GTK_WINDOW_DIALOG));
     gtk_signal_connect(GTK_OBJECT(wnd),"destroy",processdialog_destroy,NULL);
     gtk_window_set_title(wnd,"Processing data...");
     gtk_window_set_modal(wnd,TRUE);
     a = gtk_vbox_new(FALSE,0);
     gtk_container_add(GTK_CONTAINER(wnd),a);
     gtk_widget_show(a);
     b = gtk_label_new("");
     lbl = GTK_LABEL(b);
     gtk_container_add(GTK_CONTAINER(a),b);
     gtk_widget_show(b);
     b = gtk_progress_bar_new();
     bar = GTK_PROGRESS_BAR(b);
     gtk_progress_set_percentage(GTK_PROGRESS(bar),0.0);
     gtk_container_add(GTK_CONTAINER(a),b);
     gtk_widget_show(b);
     b = gtk_hseparator_new();
     gtk_container_add(GTK_CONTAINER(a),b);
     gtk_widget_show(b);
     b = gtk_button_new_with_label("Cancel");
     gtk_signal_connect_object(GTK_OBJECT(b),"clicked",
			       GTK_SIGNAL_FUNC(gtk_widget_destroy),
			       GTK_OBJECT(wnd));
     gtk_container_add(GTK_CONTAINER(a),b);
     gtk_widget_show(b);
     gtk_widget_show(GTK_WIDGET(wnd));
     window_showed = TRUE;
}

void processdialog_setup(guint32 total_samples)
{
     g_assert(wnd==NULL && !active);

     total = total_samples;
     current = 0;
     lasttime = time(0);
     accum = 0;
     tresh = total_samples / 100;
     if (tresh == 0) tresh=1;
     if (tresh > 100000) tresh=100000;
     window_showed = FALSE;
     active = TRUE;
}

gboolean processdialog_progress(guint32 num_samples)
{
     time_t t;
     char *c;
     current += num_samples;
     accum += num_samples;
     mainloop();
     if (accum < tresh) return FALSE;
     accum = 0;
     if (window_showed && !wnd) return TRUE;
     t = time(0);
     if (t!=lasttime) {
	  if (!wnd) create_wnd();
	  if (total) {
	       gtk_progress_set_percentage(GTK_PROGRESS(bar),current>total?1.0:
					   (gfloat)current/(gfloat)total);
	       c = g_strdup_printf("%ld / %ld",(long int)current,(long int)total);
	  } else {
	       c = g_strdup_printf("%ld",(long int)current);
	       gtk_progress_set_activity_mode(GTK_PROGRESS(bar),TRUE);
	       gtk_progress_set_percentage(GTK_PROGRESS(bar),0); /* ? */
	  }
	  gtk_label_set_text(lbl,c);
	  g_free(c);
	  /* lasttime = t; */
     }
     idle_work_flag=FALSE;
     while (!idle_work_flag) 
	  mainloop();
     /* idle_work(NULL); */
     return FALSE;
}

void processdialog_finished(void)
{
     if (wnd) gtk_widget_destroy(GTK_WIDGET(wnd));
     active = FALSE;
}

gboolean processdialog_active(void)
{
     return active;
}
