/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <math.h>
#include <gtk/gtk.h>
#include "mainwindow.h"
#include "sampleratedialog.h"
#include "chunk.h"
#include "float_box.h"

static void toggle(GtkToggleButton *togglebutton, gboolean *user_data)
{
     *user_data = gtk_toggle_button_get_active ( togglebutton );
}

static Chunk *apply_proc(Chunk *chunk, Mainwindow *mw, gpointer user_data)
{
     SamplerateDialog *s = SAMPLERATE_DIALOG(user_data);
     return chunk_convert_samplerate(chunk,s->rate->val,s->linear?
				     INTERPOLATION_LINEAR:INTERPOLATION_NONE);
}

static void apply(EffectDialog *ed)
{
     SamplerateDialog *s = SAMPLERATE_DIALOG(ed);
     if (intbox_check(s->rate) || s->rate->val==0) return;
     mainwindow_effect_manual(ed->mw,apply_proc,FALSE,ed);
     mainwindow_update_texts();
}

static void samplerate_dialog_class_init(EffectDialogClass *klass)
{
     klass->apply = apply;
}

static void samplerate_dialog_init(SamplerateDialog *v)
{
     EffectDialog *ed = EFFECT_DIALOG(v);
     GtkWidget *a,*b,*c;

     a = gtk_vbox_new ( FALSE, 3 );
     gtk_box_pack_start (GTK_BOX(ed->input_area),a,FALSE,FALSE,0);
     gtk_widget_show ( a );
     b = gtk_label_new ( "(This changes the sample rate of \n"
			 "the entire file, not just the selection)" );
     gtk_box_pack_start ( GTK_BOX(a),b,FALSE,FALSE,0 );
     gtk_widget_show ( b );
     b = gtk_hbox_new ( FALSE, 3 );
     gtk_box_pack_start (GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     c = gtk_label_new ( "New samplerate: " );
     gtk_box_pack_start (GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show ( c );
     c = intbox_new(0);
     v->rate = INTBOX(c);
     gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
     gtk_widget_show( c );
     b = gtk_check_button_new_with_label("Linear interpolation");
     gtk_signal_connect(GTK_OBJECT(b),"toggled",GTK_SIGNAL_FUNC(toggle),
			&(v->linear));
     gtk_box_pack_start(GTK_BOX(a),b,FALSE,FALSE,0);
     gtk_widget_show ( b );

     gtk_window_set_title(GTK_WINDOW(v),"Convert samplerate");
}

GtkType samplerate_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "SamplerateDialog",
	       sizeof(SamplerateDialog),
	       sizeof(SamplerateDialogClass),
	       (GtkClassInitFunc) samplerate_dialog_class_init,
	       (GtkObjectInitFunc) samplerate_dialog_init
	  };
	  id = gtk_type_unique(effect_dialog_get_type(),&info);
     }
     return id;
}

GtkWidget *samplerate_dialog_new(Mainwindow *mw)
{
     GtkWidget *w;
     w = gtk_type_new(samplerate_dialog_get_type());
     effect_dialog_set_mainwindow(EFFECT_DIALOG(w),mw);
     intbox_set(SAMPLERATE_DIALOG(w)->rate,mw->view->chunk->format.samplerate);
     return w;
}
