/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include <math.h>
#include <gtk/gtk.h>
#include "mainwindow.h"
#include "effectdialog.h"
#include "volumedialog.h"
#include "float_box.h"

static gdouble current_volume;
static gdouble volume_change_per_sample;
static gint current_channel;
static guint channels;

// Specialfall - konstant volym
static gboolean volume_dialog_apply_proc_0(void *in,guint sample_size,
					   chunk_writeout_func out_proc, WriteoutID id,
					   Chunk *chunk)
{
     sample_t d;
     d = *((sample_t *)in);
     d *= current_volume;
     return out_proc(id,&d,sizeof(d));
}

// En kanal
static gboolean volume_dialog_apply_proc_1(void *in,guint sample_size,
					   chunk_writeout_func out_proc, WriteoutID id,
					   Chunk *chunk)
{
     sample_t s = *((sample_t *)in);
     s *= current_volume;
     current_volume += volume_change_per_sample;
     return out_proc(id,&s,sizeof(s));
}

// Tv kanaler
static gboolean volume_dialog_apply_proc_2(void *in, guint sample_size, 
					   chunk_writeout_func out_proc, WriteoutID id,
					   Chunk *chunk)
{
     sample_t *s = (sample_t *) in;
     sample_t d[2];
     d[0] = s[0] * current_volume;
     d[1] = s[1] * current_volume;
     current_volume += volume_change_per_sample;
     return out_proc(id,d,sizeof(d));
}

// Allmnna fallet
static gboolean volume_dialog_apply_proc(void *in, guint sample_size, 
					 chunk_writeout_func out_proc, WriteoutID id,
					 Chunk *chunk)
{
     sample_t s[16], *i = (sample_t *)in;
     int k;
     for (k=0; k<channels; k++)
	  s[k] = i[k] * current_volume;
     current_volume += volume_change_per_sample;
     return out_proc(id,s,sample_size);
}

static void volume_dialog_apply(EffectDialog *ed)
{
     VolumeDialog *v = VOLUME_DIALOG(ed);
     if (floatbox_check(v->start_percent) || floatbox_check(v->end_percent))
	  return;
     current_volume = v->start_percent->val / 100.0;
     volume_change_per_sample = (v->end_percent->val / 100.0) - current_volume;
     volume_change_per_sample /= (gdouble)(mainwindow_parse_length(ed->mw));
     current_channel = 0;
     channels = ed->mw->view->chunk->format.channels;
     // printf("volume_change_per_sample = %f\n",volume_change_per_sample);
     if (volume_change_per_sample == 0.0) {
	  if (current_volume == 1.0)
	       return;
	  else
	       mainwindow_effect(ed->mw,volume_dialog_apply_proc_0,NULL,FALSE,TRUE);
     } else if (channels == 1)
	  mainwindow_effect(ed->mw,volume_dialog_apply_proc_1,NULL,TRUE,TRUE);
     else if (channels == 2)
	  mainwindow_effect(ed->mw,volume_dialog_apply_proc_2,NULL,TRUE,TRUE);
     else
	  mainwindow_effect(ed->mw,volume_dialog_apply_proc,NULL,TRUE,TRUE);
}

static sample_t peak;

static gboolean findtop_proc(void *sample, gint sample_size, Chunk *chunk)
{
     sample_t d;
     d = fabs(* ((sample_t *)sample));
     if (d>peak) peak = d;
     return (d>=1.0);
}

static void findtop(VolumeDialog *v)
{
     gfloat f;
     peak = 0.0;
     mainwindow_parse(EFFECT_DIALOG(v)->mw,findtop_proc,FALSE,TRUE);
     f = 100.0 / peak;
     floatbox_set(v->start_percent,f);
     floatbox_set(v->end_percent,f);
}

static void volume_dialog_init(VolumeDialog *v)
{
     GtkWidget *b,*c;

     b = gtk_table_new ( 2, 3, FALSE );
     gtk_box_pack_start(GTK_BOX(EFFECT_DIALOG(v)->input_area),b,FALSE,FALSE,0);
     gtk_widget_show(b);
     c = gtk_label_new ("Start:");
     gtk_label_set_justify(GTK_LABEL(c),GTK_JUSTIFY_RIGHT);
     gtk_table_attach( GTK_TABLE (b), c, 0, 1, 0, 1, 0, 0, 0, 0 );
     gtk_widget_show(c);
     c = gtk_label_new ("End:");
     gtk_label_set_justify(GTK_LABEL(c),GTK_JUSTIFY_RIGHT);
     gtk_table_attach( GTK_TABLE(b), c, 0, 1, 1, 2, 0, 0, 0, 0 );
     gtk_widget_show(c);
     c = floatbox_new(100.0);
     v->start_percent = FLOATBOX(c);
     gtk_table_attach(GTK_TABLE(b),c,1,2,0,1,
		      GTK_EXPAND|GTK_SHRINK|GTK_FILL,0,0,0);
     gtk_widget_show(c);
     c = floatbox_new(100.0);
     v->end_percent = FLOATBOX(c);
     gtk_table_attach(GTK_TABLE(b), c, 1, 2, 1, 2, 
		      GTK_EXPAND|GTK_SHRINK|GTK_FILL,0,0,0);
     gtk_widget_show(c);
     c = gtk_label_new("%");
     gtk_table_attach(GTK_TABLE(b),c,2,3,0,1,0,0,0,0);
     gtk_widget_show(c);
     c = gtk_label_new("%");
     gtk_table_attach(GTK_TABLE(b),c,2,3,1,2,0,0,0,0);
     gtk_widget_show(c);
     effect_dialog_add_button(EFFECT_DIALOG(v),"Find top volume",
			      GTK_SIGNAL_FUNC(findtop));

     gtk_window_set_title(GTK_WINDOW(v),"Set volume");
}

static void volume_dialog_class_init(EffectDialogClass *klass)
{
     klass->apply = volume_dialog_apply;
}

GtkType volume_dialog_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "VolumeDialog",
	       sizeof(VolumeDialog),
	       sizeof(VolumeDialogClass),
	       (GtkClassInitFunc) volume_dialog_class_init,
	       (GtkObjectInitFunc) volume_dialog_init
	  };
	  id = gtk_type_unique(effect_dialog_get_type(),&info);
     }
     return id;
}

GtkWidget *volume_dialog_new(Mainwindow *mw)
{
     GtkWidget *w;
     w = gtk_type_new(volume_dialog_get_type());
     effect_dialog_set_mainwindow(EFFECT_DIALOG(w),mw);
     return w;
}
