/*
 * Copyright (C) 2002 2003, Magnus Hjorth
 *
 * This file is part of mhWaveEdit.
 *
 * mhWaveEdit is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by        
 * the Free Software Foundation; either version 2 of the License, or  
 * (at your option) any later version.
 *
 * mhWaveEdit is distributed in the hope that it will be useful,   
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with mhWaveEdit; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 */


#include "vu_meter.h"
#include "main.h"

static void vu_meter_size_request(GtkWidget *widget, GtkRequisition *req)
{
     req->width = 250;
     req->height = 8;
}

static gint vu_meter_expose(GtkWidget *widget, GdkEventExpose *event)
{
     VuMeter *vu = VU_METER(widget);
     gfloat green_part,red_part,black_part;
     gint x=0,pw,w,h;

     if (vu->value > vu->red_limit) {
	  green_part = vu->red_limit;
	  red_part = vu->value - vu->red_limit;
     } else {
	  green_part = vu->value;
	  red_part = 0.0;
     }
     black_part = 1.0 - green_part - red_part;

     w = widget->allocation.width;
     h = widget->allocation.height;

     pw = (gint)(green_part * (gfloat)w);
     if (pw>0) 
	  gdk_draw_rectangle(widget->window,gc_green(widget),TRUE,x,0,pw,h);
     x += pw;
     
     pw = (gint)(red_part * (gfloat)w);
     if (pw>0)
	  gdk_draw_rectangle(widget->window,gc_red(widget),TRUE,x,0,pw,h);
     x += pw;

     if (black_part > 0.0)
	  gdk_draw_rectangle(widget->window,gc_black(widget),TRUE,x,0,w-x,h);
     
     x = MIN(((float)w * vu->peak),w-4);
     gdk_draw_rectangle(widget->window,gc_grey(widget),TRUE,x,0,4,h);

     return TRUE;
}

static void vu_meter_class_init(GtkWidgetClass *klass)
{
     klass->size_request = vu_meter_size_request;
     klass->expose_event = vu_meter_expose;
}

static void vu_meter_init(VuMeter *v)
{
     v->value = 0.0;
     v->red_limit = 1.0;
     v->peak = 0.0;
}

GtkType vu_meter_get_type(void)
{
     static GtkType id = 0;
     if (!id) {
	  GtkTypeInfo info = {
	       "VuMeter",
	       sizeof(VuMeter),
	       sizeof(VuMeterClass),
	       (GtkClassInitFunc) vu_meter_class_init,
	       (GtkObjectInitFunc) vu_meter_init
	  };
	  id = gtk_type_unique(gtk_drawing_area_get_type(),&info);
     }
     return id;
}

GtkWidget *vu_meter_new(gfloat value, gfloat red_limit)
{
     VuMeter *v;
     v = gtk_type_new(vu_meter_get_type());
     v->value = value;
     v->red_limit = red_limit;
     return GTK_WIDGET(v);
}

void vu_meter_set_value(VuMeter *v, gfloat value)
{
     GTimeVal tv,tv2;
     v->value = value;
     if (value > v->peak) {
	  v->peak = value;
	  g_get_current_time(&(v->peaktime));
     } else {
	  g_get_current_time(&tv);
	  timeval_subtract(&tv2,&tv,&(v->peaktime));
	  if (tv2.tv_usec > 500000 || tv2.tv_sec > 0) {
	       v->peak = value;
	       g_get_current_time(&(v->peaktime));
	  }
     }
     gtk_widget_queue_draw(GTK_WIDGET(v));
}

void vu_meter_set_red_limit(VuMeter *v, gfloat limit)
{
     v->red_limit = limit;
     gtk_widget_queue_draw(GTK_WIDGET(v));
}
